#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  validators.py
"""
Various validator classes
"""
#
#  Copyright 2019-2020 Dominic Davis-Foster <dominic@davis-foster.co.uk>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#

# stdlib
import string

# 3rd party
import wx


class ValidatorBase(wx.Validator):
	"""
	Based on the wxPython Demo
	Licenced under the wxWindows Library Licence, Version 3.1
	"""
	
	def Clone(self):
		"""
		Standard cloner.

		Note that every validator must implement the Clone() method.
		"""
		return self.__class__()
	
	def TransferToWindow(self):
		""" Transfer data from validator to window.

			The default implementation returns False, indicating that an error
			occurred.  We simply return True, as we don't do any data transfer.
		"""
		return True  # Prevent wxDialog from complaining.
	
	def TransferFromWindow(self):
		""" Transfer data from window to validator.

			The default implementation returns False, indicating that an error
			occurred.  We simply return True, as we don't do any data transfer.
		"""
		return True  # Prevent wxDialog from complaining.
	
	def set_warning(self):
		self.GetWindow().SetBackgroundColour("pink")
		self.GetWindow().SetFocus()
		self.GetWindow().Refresh()
		return False
	
	def reset_ctrl(self):
		self.GetWindow().SetBackgroundColour(
				wx.SystemSettings.GetColour(wx.SYS_COLOUR_WINDOW))
		self.GetWindow().Refresh()
		return True


class CharValidator(ValidatorBase):
	"""
	A Validator that only allows the type of characters selected to be entered.
	
	The possible flags are:
		int-only - only the numbers 0123456789 can be entered
		float-only - only numbers and decimal points can be entered
	
	Based on http://wxpython-users.1045709.n5.nabble.com/best-way-to-restrict-input-to-integers-td2370605.html
	"""
	
	def __init__(self, flag):
		wx.Validator.__init__(self)
		self.flag = flag
		self.Bind(wx.EVT_CHAR, self.OnChar)
		
		# Special allowed keys, including del, Ctrl+C, Ctrl+X and Ctrl+V
		self.special_keys = {
				wx.WXK_BACK,
				wx.WXK_TAB,
				wx.WXK_RETURN,
				wx.WXK_SHIFT,
				wx.WXK_CONTROL,
				wx.WXK_ALT,
				wx.WXK_CONTROL_C,
				wx.WXK_CONTROL_V,
				wx.WXK_CONTROL_X,
				wx.WXK_DELETE,
				}
		
	def Clone(self):
		"""
		Standard cloner.

		Note that every validator must implement the Clone() method.
		"""
		return self.__class__(self.flag)
	
	def Validate(self, win):
		return True
	
	def OnChar(self, event):
		keycode = int(event.GetKeyCode())
		if keycode < 256:
			key = chr(keycode)
			
			if keycode in self.special_keys:
				event.Skip()
			if self.flag == "int-only":
				if key not in "0123456789":
					return
			elif self.flag == "float-only":
				if key == "." and "." in event.GetEventObject().GetValue():
					return
				if key not in "0123456789.":
					return
			elif self.flag == 'no-alpha' and key in string.ascii_letters:
				return
			elif self.flag == 'no-digit' and key in string.digits:
				return
		
		event.Skip()


class FloatValidator(CharValidator):
	"""
	A Validator that only allows numbers and decimal points to be entered.
	If a decimal point has already been entered, a second one cannot be entered.
	The argument `flag` is used to limit the number of decimal places that can be entered.
	"""
	def OnChar(self, event):
		
		keycode = int(event.GetKeyCode())
		if keycode < 256:
			key = chr(keycode)
			if keycode in self.special_keys:
				event.Skip()
			
			current_value = event.GetEventObject().GetValue()
			
			if key not in "0123456789.":
				return
			
			if "." in current_value:
				if key == ".":
					if "." not in event.GetEventObject().GetStringSelection():
						return
				
				# Current decimal places
				current_dp = len(current_value.split(".")[1])
				
				if current_dp == self.flag:
					# Already at maximum decimal places
					return
		
		event.Skip()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  StylePickerPanel.py
#
#  Copyright 2019 Dominic Davis-Foster <dominic@davis-foster.co.uk>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#
#
# generated by wxGlade 0.9.3 on Tue Apr  9 18:07:58 2019
#

# stdlib
import sys

# 3rd party
import wx
from matplotlib.backends.backend_wxagg import FigureCanvasWxAgg as FigureCanvas
from matplotlib.figure import Figure

# begin wxGlade: dependencies
# end wxGlade

# begin wxGlade: extracode
# end wxGlade


default_styles = ["o", "v", "^", "<", ">", "s", "h", "X", "D", "d"]


class StylePickerPanel(wx.Panel):
	def __init__(
			self, parent, id=wx.ID_ANY, pos=wx.DefaultPosition, size=wx.DefaultSize,
			style=wx.TAB_TRAVERSAL, name=wx.PanelNameStr, label="Choose Styles: ",
			selection_choices=None,
			):
		if selection_choices is None:
			selection_choices = default_styles[:]
		self.label = label
		self.selection_choices = selection_choices

		args = (parent, id, pos, size)
		kwds = {"style": style, "name": name}
		
		# begin wxGlade: StylePickerPanel.__init__
		kwds["style"] = kwds.get("style", 0) | wx.TAB_TRAVERSAL
		wx.Panel.__init__(self, *args, **kwds)
		self.main_panel = wx.Panel(self, wx.ID_ANY)
		self.move_panel = wx.Panel(self.main_panel, wx.ID_ANY)
		self.picker_list_box = wx.ListBox(self.main_panel, wx.ID_ANY, choices=[])
		
		self.picker_figure = Figure()
		
		self.picker_canvas = FigureCanvas(self.main_panel, wx.ID_ANY, self.picker_figure)
		self.add_btn = wx.Button(self.main_panel, wx.ID_ANY, u"Add 🡲")
		self.remove_btn = wx.Button(self.main_panel, wx.ID_ANY, u"🡰 Remove")
		self.selection_list_box = wx.ListBox(self.main_panel, wx.ID_ANY, choices=[])
		self.up_btn = wx.Button(self.main_panel, wx.ID_ANY, u"🡱 Up")
		self.down_btn = wx.Button(self.main_panel, wx.ID_ANY, u"🡳 Down")
		
		self.selection_figure = Figure()
		
		self.selection_canvas = FigureCanvas(self.main_panel, wx.ID_ANY, self.selection_figure)

		self.__set_properties()
		self.__do_layout()

		self.Bind(wx.EVT_LISTBOX, self.update_picker_preview, self.picker_list_box)
		self.Bind(wx.EVT_LISTBOX_DCLICK, self.add, self.picker_list_box)
		self.Bind(wx.EVT_BUTTON, self.add, self.add_btn)
		self.Bind(wx.EVT_BUTTON, self.remove, self.remove_btn)
		self.Bind(wx.EVT_LISTBOX, self.update_selection_preview, self.selection_list_box)
		self.Bind(wx.EVT_BUTTON, self.move_up, self.up_btn)
		self.Bind(wx.EVT_BUTTON, self.move_down, self.down_btn)
		# end wxGlade
	
		self.Bind(wx.EVT_LISTBOX_DCLICK, self.remove, self.selection_list_box)
		
		self.markers = {
				"point": ".",
				"pixel": ",",
				"circle": "o",
				"triangle_down": "v",
				"triangle_up": "^",
				"triangle_left": "<",
				"triangle_right": ">",
				"tri_down": "1",
				"tri_up": "2",
				"tri_left": "3",
				"tri_right": "4",
				"octagon": "8",
				"square": "s",
				"pentagon": "p",
				"plus (filled)": "P",
				"star": "*",
				"hexagon1": "h",
				"hexagon2": "H",
				"plus": "+",
				"x": "x",
				"x (filled)": "X",
				"diamond": "D",
				"thin_diamond": "d",
				"caretleft": 4,  # (CARETLEFT),
				"caretright": 5,  # (CARETRIGHT),
				"caretup": 6,  # (CARETUP),
				"caretdown": 7,  # (CARETDOWN),
				}
		
		for marker in self.selection_choices:
			for key in self.markers.keys():
				if marker == self.markers[key]:
					self.selection_list_box.Append(key)
		if not self.selection_list_box.IsEmpty():
			self.selection_list_box.SetSelection(0)
		
		for marker in list(filter(lambda x: x not in self.selection_choices, [self.markers[y] for y in self.markers])):
			for key in self.markers.keys():
				if marker == self.markers[key]:
					self.picker_list_box.Append(key)
		self.picker_list_box.SetSelection(0)
		
		self.picker_axes = self.picker_figure.add_subplot(111)
		self.selection_axes = self.selection_figure.add_subplot(111)
		self.update_picker_preview()
		self.update_selection_preview()

	def __set_properties(self):
		# begin wxGlade: StylePickerPanel.__set_properties
		self.move_panel.SetMinSize((170, -1))
		self.picker_list_box.SetMinSize((170, 256))
		self.picker_canvas.SetMinSize((64, 64))
		self.selection_list_box.SetMinSize((170, 256))
		self.up_btn.SetMinSize((80, -1))
		self.down_btn.SetMinSize((80, -1))
		self.selection_canvas.SetMinSize((64, 64))
		self.main_panel.SetMinSize((450, -1))
		# end wxGlade

	def __do_layout(self):
		# begin wxGlade: StylePickerPanel.__do_layout
		parent_sizer = wx.BoxSizer(wx.HORIZONTAL)
		main_sizer = wx.BoxSizer(wx.VERTICAL)
		list_grid_sizer = wx.BoxSizer(wx.HORIZONTAL)
		sizer_4 = wx.BoxSizer(wx.VERTICAL)
		selection_preview_sizer = wx.BoxSizer(wx.HORIZONTAL)
		move_grid = wx.GridSizer(1, 2, 0, 5)
		sizer_3 = wx.BoxSizer(wx.VERTICAL)
		sizer_2 = wx.BoxSizer(wx.VERTICAL)
		picker_preview_sizer = wx.BoxSizer(wx.HORIZONTAL)
		grid_sizer = wx.GridSizer(1, 3, 10, 10)
		borders_label = wx.StaticText(self.main_panel, wx.ID_ANY, "Choose Styles: ")
		borders_label.SetMinSize((128, 20))
		borders_label.SetFont(wx.Font(12, wx.FONTFAMILY_DEFAULT, wx.FONTSTYLE_NORMAL, wx.FONTWEIGHT_BOLD, 0, ""))
		grid_sizer.Add(borders_label, 0, wx.BOTTOM, 7)
		grid_sizer.Add(self.move_panel, 1, wx.ALIGN_CENTER | wx.EXPAND, 0)
		main_sizer.Add(grid_sizer, 0, 0, 0)
		sizer_2.Add(self.picker_list_box, 5, wx.EXPAND, 0)
		sizer_2.Add((0, 0), 0, 0, 0)
		preview_label = wx.StaticText(self.main_panel, wx.ID_ANY, "Preview: ")
		picker_preview_sizer.Add(preview_label, 0, 0, 0)
		picker_preview_sizer.Add(self.picker_canvas, 1, wx.EXPAND, 0)
		sizer_2.Add(picker_preview_sizer, 0, wx.EXPAND | wx.TOP, 5)
		list_grid_sizer.Add(sizer_2, 1, wx.EXPAND, 0)
		sizer_3.Add(self.add_btn, 0, wx.ALIGN_CENTER | wx.BOTTOM, 10)
		sizer_3.Add(self.remove_btn, 0, wx.ALIGN_CENTER, 0)
		list_grid_sizer.Add(sizer_3, 5, wx.ALIGN_CENTER | wx.BOTTOM, 64)
		sizer_4.Add(self.selection_list_box, 5, wx.EXPAND, 0)
		move_grid.Add(self.up_btn, 0, wx.ALIGN_CENTER, 0)
		move_grid.Add(self.down_btn, 0, wx.ALIGN_CENTER, 0)
		sizer_4.Add(move_grid, 0, wx.EXPAND, 0)
		preview_label_1 = wx.StaticText(self.main_panel, wx.ID_ANY, "Preview: ")
		selection_preview_sizer.Add(preview_label_1, 0, 0, 0)
		selection_preview_sizer.Add(self.selection_canvas, 1, wx.EXPAND, 0)
		sizer_4.Add(selection_preview_sizer, 0, wx.EXPAND | wx.TOP, 5)
		list_grid_sizer.Add(sizer_4, 1, wx.EXPAND, 0)
		main_sizer.Add(list_grid_sizer, 0, 0, 0)
		static_line_11 = wx.StaticLine(self.main_panel, wx.ID_ANY)
		main_sizer.Add(static_line_11, 0, wx.BOTTOM | wx.EXPAND | wx.TOP, 5)
		self.main_panel.SetSizer(main_sizer)
		parent_sizer.Add(self.main_panel, 0, wx.ALL, 10)
		self.SetSizer(parent_sizer)
		parent_sizer.Fit(self)
		self.Layout()
		# end wxGlade
		borders_label.SetLabel(self.label)
	
	def move_up(self, event):  # wxGlade: StylePickerPanel.<event_handler>
		self.move(-1)
		event.Skip()
	
	def move_down(self, event):  # wxGlade: StylePickerPanel.<event_handler>
		self.move(1)
		event.Skip()
	
	def move(self, direction=1):
		selection = self.selection_list_box.GetSelection()
		selection_string = self.selection_list_box.GetString(selection)
		if self.selection_list_box.GetCount() == selection + direction or selection + direction < 0:
			return
		
		self.selection_list_box.Delete(selection)
		self.selection_list_box.InsertItems([selection_string], selection + direction)
		self.selection_list_box.SetSelection(selection + direction)
	
	def add(self, event):  # wxGlade: StylePickerPanel.<event_handler>
		selection = self.picker_list_box.GetSelection()
		if selection == -1:
			return
		
		selection_string = self.picker_list_box.GetString(selection)
		if selection_string == '':
			return
		
		self.selection_list_box.Append(selection_string)
		self.picker_list_box.Delete(selection)
		
		self.update_picker_preview()
		event.Skip()
	
	def remove(self, event):  # wxGlade: StylePickerPanel.<event_handler>
		selection = self.selection_list_box.GetSelection()
		if selection == -1:
			return
		
		selection_string = self.selection_list_box.GetString(selection)
		if selection_string == '':
			return
		
		self.picker_list_box.Append(selection_string)
		self.selection_list_box.Delete(self.selection_list_box.GetSelection())
		
		self.update_selection_preview()
		event.Skip()
	
	def update_picker_preview(self, *_):  # wxGlade: StylePickerPanel.<event_handler>
		self.update_preview(self.picker_list_box, self.picker_axes)
		self.picker_canvas.draw_idle()
	
	def update_selection_preview(self, *_):  # wxGlade: StylePickerPanel.<event_handler>
		self.update_preview(self.selection_list_box, self.selection_axes)
		self.selection_canvas.draw_idle()
	
	def update_preview(self, list_obj, axes):
		axes.clear()
		axes.axis('off')
		selection_string = list_obj.GetStringSelection()
		if selection_string == '':
			return
		
		axes.scatter(1, 1, s=200, color="red", marker=self.markers[selection_string])
	
	do_layout = __do_layout
	set_properties = __set_properties
	
	def get_selection(self):
		return [
				self.markers[
					self.selection_list_box.GetString(item)
					] for item in range(self.selection_list_box.GetCount())
				]
	
	def _do_layout(self):
		return self.__do_layout()
	
	def _set_properties(self):
		return self.__set_properties()

# end of class StylePickerPanel



#  !/usr/bin/env python
#   -*- coding: utf-8 -*-
#
#  imagepanel.py
"""
Based on ChartPanelBase, a canvas for displaying an image within a wxPython window
using PIL and matplotlib, with a right click menu with some basic options
"""
#
#  Copyright 2020 Dominic Davis-Foster <dominic@davis-foster.co.uk>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#

# 3rd party
import matplotlib
import wx
from matplotlib.figure import Figure
from PIL import Image

# this package
from domdf_wxpython_tools.chartpanel import ChartPanelBase
from domdf_wxpython_tools.dialogs import file_dialog_wildcard, Wildcards
from domdf_wxpython_tools.projections import NoZoom

# Prevent zooming of axis with mouse click
matplotlib.projections.register_projection(NoZoom)

# Wildcard for Open and Save dialogs
images_wildcard = Wildcards()
images_wildcard.add_image_wildcard()
images_wildcard.add_common_filetype("jpeg")
images_wildcard.add_common_filetype("png")
images_wildcard.add_common_filetype("bmp")
images_wildcard.add_common_filetype("tiff")
images_wildcard.add_common_filetype("gif")
images_wildcard.add_all_files_wildcard()

# IDs
ID_ImagePanel_Reset_View = wx.NewIdRef()
ID_ImagePanel_Copy_Image = wx.NewIdRef()
ID_ImagePanel_Paste_Image = wx.NewIdRef()
ID_ImagePanel_Save_Image = wx.NewIdRef()
ID_ImagePanel_Load_Image = wx.NewIdRef()
ID_ImagePanel_Delete_Image = wx.NewIdRef()

# Events
ImgPanelChangedEvent = wx.NewEventType()
EVT_IMAGE_PANEL_CHANGED = wx.PyEventBinder(ImgPanelChangedEvent, 0)


class EvtImgPanelChanged(wx.PyCommandEvent):
	"""
	Custom Event for an image in the ImagePanel being changed
	"""
	eventType = ImgPanelChangedEvent

	def __init__(self, windowID, obj):
		"""
		
		:param windowID:
		:type windowID: int
		:param obj:
		:type obj:
		"""
		
		wx.PyCommandEvent.__init__(self, self.eventType, windowID)
		self.SetEventObject(obj)


class ImagePanel(ChartPanelBase):
	"""
	Panel that contains a matplotlib plotting window, used for displaying an image.
	The image can be right clicked to bring up a context menu allowing copying, pasting and saving of the image.
	The image can be panned by holding the left mouse button and moving the mouse,
	and zoomed in and out using the scrollwheel on the mouse.
	"""
	
	default_image = ("RGB", (640, 480), (240, 240, 240))
	
	def __init__(
			self, parent, image=None, id=wx.ID_ANY, pos=wx.DefaultPosition,
			size=wx.DefaultSize, style=0, name=wx.PanelNameStr):
		"""
		:param parent: The parent window.
		:type parent: wx.Window
		:param image:
		:type image:
		:param id: An identifier for the panel. wx.ID_ANY is taken to mean a default.
		:type id: wx.WindowID, optional
		:param pos: The panel position. The value ::wxDefaultPosition indicates a default position,
		:type pos: wx.Point, optional
			chosen by either the windowing system or wxWidgets, depending on platform.
		:param size: The panel size. The value ::wxDefaultSize indicates a default size, chosen by
		:type size: wx.Size, optional
			either the windowing system or wxWidgets, depending on platform.
		:param style: The window style. See wxPanel.
		:type style: int, optional
		:param name: Window name.
		:type name: str, optional
		"""
		
		fig = Figure()
		ax = fig.add_subplot(111, frameon=False, projection="NoZoom")  # 1x1 grid, first subplot
		
		ChartPanelBase.__init__(self, parent, fig, ax, id, pos, size, style, name)
		
		if isinstance(image, Image.Image):
			# PIL Image object, load directly
			self._image = image
		elif image is None:
			self._image = Image.new(*self.default_image)
		else:
			# Filename, load from file
			# self._image = mpimg.imread(image)
			self._image = Image.open(image)
			
		self.editable = True
		
		self._setup_context_menu()
		
		self._load_image()
		wx.CallAfter(self.reset_view)
	
	def _setup_context_menu(self):
		self.context_menu = wx.Menu()
		
		self.context_menu.Append(ID_ImagePanel_Reset_View, "Reset View")
		self.Bind(wx.EVT_MENU, self.reset_view, id=ID_ImagePanel_Reset_View)
		
		self.context_menu.AppendSeparator()
		
		self.context_menu.Append(ID_ImagePanel_Copy_Image, "Copy Image")
		self.Bind(wx.EVT_MENU, self.copy, id=ID_ImagePanel_Copy_Image)
		self.context_menu.Append(ID_ImagePanel_Paste_Image, "Paste Image")
		self.Bind(wx.EVT_MENU, self.paste, id=ID_ImagePanel_Paste_Image)
		
		self.context_menu.Append(ID_ImagePanel_Save_Image, "Save Image")
		self.Bind(wx.EVT_MENU, self.on_save, id=ID_ImagePanel_Save_Image)
		
		self.context_menu.AppendSeparator()
		
		self.context_menu.Append(ID_ImagePanel_Load_Image, "Load Image")
		self.Bind(wx.EVT_MENU, self.on_load, id=ID_ImagePanel_Load_Image)
		
		self.context_menu.Append(ID_ImagePanel_Delete_Image, "Delete Image")
		self.Bind(wx.EVT_MENU, self.clear, id=ID_ImagePanel_Delete_Image)
	
	def load_image(self, new_image=None, suppress_event=False):
		"""
		Load the 'new_image' into the contol
		
		:param new_image: The image to load, or a string pointing to the image on a filesystem
		:type new_image: :class:`PIL.Image.Image` or str or pathlib.Path, optional
		:param suppress_event: Whether the event that the image has changed should be suppressed, default False
		:type suppress_event: bool, optional
		
		TODO: Do we even need to be triggering an event here, since "load_image"
		 is only ever going to be called by the programmer
		"""
		
		self.ax.clear()
		self._image = None
		self._image = None
		
		if isinstance(new_image, Image.Image):
			# PIL Image object, load directly
			self._image = new_image
		elif new_image is None:
			self._image = Image.new(*self.default_image)
		else:
			# Filename, load from file
			# self._image = mpimg.imread(image)
			self._image = Image.open(str(new_image))
		
		self._load_image()
		self.pan(True)
		
		if not suppress_event:
			wx.PostEvent(self.GetEventHandler(), EvtImgPanelChanged(self.GetId(), self))
	
	def _load_image(self):
		"""
		Internal function for the actual loading of the image
		"""
		
		self.ax.clear()
		self.ax.imshow(self._image, aspect='equal')
		
		self.ax.axes.get_xaxis().set_visible(False)
		self.ax.axes.get_yaxis().set_visible(False)
		# self.fig.tight_layout()
		self.fig.subplots_adjust(left=0, bottom=0, top=1, right=1)
		
		self.canvas.draw()
		
		self.pan(True)
		self.ax.autoscale(tight=True)
		self.setup_scrollwheel_zooming()
		self.canvas.mpl_connect('button_press_event', self.on_context_menu)
		
	def on_context_menu(self, event):
		"""
		Event Handler for bringing up right click context menu
		"""
		
		if event.button == matplotlib.backend_bases.MouseButton.RIGHT:
			event.guiEvent.GetEventObject().ReleaseMouse()
			print('in context_menu callback: clicked at (%g, %g)' % (event.x, event.y))
			self.PopupMenu(self.context_menu)
			# UIActionSimulator().MouseClick(wx.MOUSE_BTN_RIGHT)
	
	def copy(self, event=None):
		"""
		Copy the image to the clipboard
		"""
		
		width, height = self._image.size
		bmp = wx.Bitmap.FromBuffer(width, height, self._image.tobytes())
		
		# Create BitmapDataObject
		bmp_data = wx.BitmapDataObject(bmp)
		
		# Write image from clipboard
		if wx.TheClipboard.Open():
			wx.TheClipboard.SetData(bmp_data)
			wx.TheClipboard.Flush()
		wx.TheClipboard.Close()
	
	def paste(self, event=None):
		"""
		Paste the image from the clipboard into the control
		"""
		
		# Create empty BitmapDataObject
		bmp_data = wx.BitmapDataObject()
		
		# Read image from clipboard
		wx.TheClipboard.Open()
		if wx.TheClipboard.GetData(bmp_data):
			wx.TheClipboard.Close()
			
			# https://stackoverflow.com/a/46606553/3092681
			# Get bitmap and convert to PIL Image
			bmp = bmp_data.GetBitmap()
			size = tuple(bmp.GetSize())
			buf = size[0] * size[1] * 3 * b"\x00"
			bmp.CopyToBuffer(buf)
			self._image = Image.frombuffer("RGB", size, buf, "raw", "RGB", 0, 1)
			self._load_image()
			self.pan(True)
			if event:
				event.Skip()
			
			wx.PostEvent(self.GetEventHandler(), EvtImgPanelChanged(self.GetId(), self))
		
		else:
			# No image on clipboard
			wx.TheClipboard.Close()
	
	def on_save(self, event=None):
		"""
		Save the image to the location selected in the dialog
		"""

		save_location = file_dialog_wildcard(
				self, "Save Image",
				images_wildcard.wildcard,
				style=wx.FD_SAVE | wx.FD_OVERWRITE_PROMPT
				)
		
		if not save_location:
			return
		
		self._image.save(save_location[0])
	
	def on_load(self, event=None):
		"""
		Load the image into the dialog from the file selected in the dialog
		"""
		
		new_image = file_dialog_wildcard(
				self, "Choose an Image",
				images_wildcard.wildcard,
				style=wx.FD_OPEN | wx.FD_FILE_MUST_EXIST
				)
		
		if not new_image:
			return
		
		self._image = Image.open(new_image[0])
		self._load_image()
		self.pan(True)
		if event:
			event.Skip()
		
		wx.PostEvent(self.GetEventHandler(), EvtImgPanelChanged(self.GetId(), self))
	
	def clear(self, event=None):
		"""
		Clear the image from the control
		"""
		
		self.ax.clear()
		self._image = None
		self._image = None
		if event:
			event.Skip()
		
		wx.PostEvent(self.GetEventHandler(), EvtImgPanelChanged(self.GetId(), self))
		
	def reset_view(self, *_):
		"""
		Reset the view of the image
		"""
		
		self._load_image()
		# self.fig.tight_layout()
		self.fig.subplots_adjust(left=0, bottom=0, top=1, right=1)
		self.canvas.SetSize(self.GetSize())
		self.Refresh()
		self.canvas.draw()
		self.canvas.Refresh()
		self.pan(True)
		# wx.CallAfter(self.pan)
	
	@property
	def image(self):
		"""
		Returns the image being displayed in the control
		
		:rtype: PIL.Image.Image
		"""
		return self._image
	#
	# @image.setter
	# def image(self, new_image):
	#
	# 	if isinstance(new_image, Image.Image):
	# 		# PIL Image object, load directly
	# 		self._image = new_image
	# 	elif new_image is None:
	# 		self._image = Image.new("RGB", (640, 480), (73, 109, 137))
	# 	else:
	# 		# Filename, load from file
	# 		# self._image = mpimg.imread(image)
	# 		self._image = Image.open(new_image)
	#
	# 	self.load_image()
	# 	#self.reset_view()
	# 	wx.CallAfter(self.reset_view)




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  icons.py
#
#  Copyright 2019 Dominic Davis-Foster <dominic@davis-foster.co.uk>
#
#  GetStockBitmap and GetStockToolbarBitmap from
#  		https://sourceforge.net/p/wxglade/mailman/message/6475744/
#       Copyright (C) 2005 Antoine Pitrou
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#

import wx


def get_toolbar_icon(icon_name, size=24):
	return wx.Bitmap(wx.ArtProvider.GetBitmap(f"wx{icon_name}", "wxART_TOOLBAR_C", wx.Size(size, size)))


def get_button_icon(icon_name, size=24):
	return wx.Bitmap(wx.ArtProvider.GetBitmap(f"wx{icon_name}", "wxART_BUTTON_C", wx.Size(size, size)))

# The following code (C) 2005 Antoine Pitrou
# https://sourceforge.net/p/wxglade/mailman/message/6475744/
_art_provider = None


def GetStockBitmap(art_id, art_client=None):
	"""
	Get a stock bitmap from its wx.ART_xxx ID
	"""
	global _art_provider
	if _art_provider is None:
		_art_provider = wx.ArtProvider()
	return _art_provider.GetBitmap(id=art_id,
								   client=art_client or wx.ART_OTHER)


def GetStockToolbarBitmap(art_id):
	return GetStockBitmap(art_id, wx.ART_TOOLBAR)


TB = GetStockToolbarBitmap



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  font_parser.py
#
#  Copyright (c) 2020 Dominic Davis-Foster <dominic@davis-foster.co.uk>
#
#  Function 'freezeargs' Copyright 2018 Cedar
#  https://stackoverflow.com/users/5810747/cedar
#  From https://stackoverflow.com/a/53394430
#  Licensed under CC-BY-SA 4.0
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#
# generated by wxGlade 0.9.2 on Thu Jan 16 16:34:51 2020
#

# stdlib
import re
from functools import lru_cache, wraps

# 3rd party
from frozendict import frozendict


def freezeargs(func):
	"""
	Make mutable dictionary immutable for lru_cache

	From stackoverflow.com/a/53394430
	"""
	
	@wraps(func)
	def wrapped(*args, **kwargs):
		args = tuple(
				[frozendict(arg) if isinstance(arg, dict) else arg for arg in args])
		kwargs = {k: frozendict(v) if isinstance(v, dict) else v for k, v in kwargs.items()}
		return func(*args, **kwargs)
	
	return wrapped


@freezeargs
@lru_cache(5)
def parse_font(style_dict):
	"""
	Parse the font from the style_dict
	
	:param style_dict: Dictionary containing styling information for the font
	:type style_dict: dict
	
	:return: Tuple containing the colour of the font, and the font properties
	:rtype: (str, dict)
	
	The font properties dictionary returned will contain the following keys:
	
		family (wx.FontFamily) – The font family: a generic portable way of referring to fonts without specifying a facename. This parameter must be one of the wx.FontFamily enumeration values. If the faceName argument is provided, then it overrides the font family.
		
		style (wx.FontStyle) – One of wx.FONTSTYLE_NORMAL, wx.FONTSTYLE_SLANT and wx.FONTSTYLE_ITALIC.
		
		weight (wx.FontWeight) – Font weight, sometimes also referred to as font boldness. One of the wx.FontWeight enumeration values.
		
		underline (wx.bool) – The value can be True or False. At present this has an effect on Windows and Motif 2.x only.
		
		faceName (wx.string) – An optional string specifying the face name to be used. If it is an empty string, a default face name will be chosen based on the family.
		
		encoding (wx.FontEncoding) – An encoding which may be one of the enumeration values of wx.FontEncoding. If the specified encoding isn’t available, no font is created (see also Font Encodings).
	
	and one of:
	
		pointSize (int) – Size in points. See SetPointSize for more info. Notice that, for historical reasons, the value 70 here is interpreted at DEFAULT and results in creation of the font with the default size and not of a font with the size of 70pt. If you really need the latter, please use SetPointSize(70). Note that this constructor and the matching Create() method overload are the only places in wx.Font API handling DEFAULT specially: neither SetPointSize nor the constructor taking wx.FontInfo handle this value in this way.
		
		pixelSize (wx.Size) – Size in pixels. See SetPixelSize for more info.

	depending on the font size specified in 'style_dict'
	"""
	
	colour = style_dict["color"]
	
	font_data = {
			"family": style_dict["font-family"],
			"style": style_dict["font-style"],
			"weight": style_dict["font-weight"],
			"faceName": style_dict["font-face-name"]
			}
	
	if style_dict["text-decoration"].lower() == "underline":
		font_data["underline"] = True
	
	# if style_dict["font-size"] is None:
	# 	font_data["pointSize"] = None
	# else:
	font_size_value, pt_or_px = filter(None, re.split("(\d+|\D+)", style_dict["font-size"]))

	if pt_or_px.lower() == "pt":
		font_data["pointSize"] = int(font_size_value)
	elif pt_or_px.lower() == "px":
		font_data["pixelSize"] = int(font_size_value)

	return colour, font_data



#!/usr/bin/env python
#   -*- coding: utf-8 -*-
#
#  FileBrowseCtrl.py
#
#  Copyright (c) 2019-2020  Dominic Davis-Foster <dominic@davis-foster.co.uk>
#  Adapted from wx.lib.filebrowsebutton.FileBrowseButton.
#  Original header below:
# ----------------------------------------------------------------------
# Name:        FileBrowseCtrl
# Purpose:     Composite controls that provide a Browse button next to
#              either a ClearableTextCtrl or a wxComboBox.  The Browse button
#              launches a wxFileDialog and loads the result into the
#              other control.
#
# Author:      Mike Fletcher, Dominic Davis-Foster
#
# Copyright:   (c) 2000-2018 by Total Control Software
# Licence:     wxWindows license
# Tags:        phoenix-port
# ----------------------------------------------------------------------
# 12/02/2003 - Jeff Grimmett (grimmtooth@softhome.net)
#
# o 2.5 Compatibility changes
#
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#


# stdlib
import os
import pathlib

# 3rd party
import wx
from wx.lib.filebrowsebutton import FileBrowseButton

# this package
# from . import icons
from domdf_wxpython_tools.dialogs import file_dialog_wildcard
from domdf_wxpython_tools.clearable_textctrl import ClearableTextCtrl

# ----------------------------------------------------------------------

# TODO: validate filetypes
# TODO: Change SearchCtrl cancel button to something more resembling clearing the control. DDF 08/01/2020

# TODO: Control doesn't indicate when it has focus; on GTK there should be an orange border but there isn't


class FileBrowseCtrl(FileBrowseButton):
	"""
	A control to allow the user to type in a filename or browse with
	the standard file dialog to select file.

	Based on and subclassed from wx.lib.filebrowsebutton.FileBrowseButton but
	with a wx.SearchCtrl in place of the wx.TextCtrl to provide the cancel/clear
	button and with an icon on the browse button.
	"""
	
	def __init__(
			self, parent, id=wx.ID_ANY, pos=wx.DefaultPosition, size=wx.DefaultSize,
			style=wx.TAB_TRAVERSAL | wx.FD_DEFAULT_STYLE, labelText="File Entry:", buttonText="Browse",
			toolTip="Type a filename or click the browse button to choose a file",
			# following are the values for a file dialog box
			dialogTitle="Choose a file", initialValue="",
			# callback for when value changes (optional)
			changeCallback=lambda x: x,
			labelWidth=0, name='fileBrowseButton', show_cancel_btn=True,
			fileMask="All files (*.*)|*.*",
			dialog_title="File Picker", **kwargs
			):
		"""
		:param parent:			Parent window. Should not be None.
		:type parent:			wx.Window
		:param id:				Control identifier. A value of -1 denotes a default value.
		:type id:				wx.WindowID
		:param pos:				Control position
		:type pos:				wx.Point
		:param size:			Control size
		:type size:				wx.Size
		:param style:			Window style. See wx.Window and ClearableTextCtrl for supported styles
		:type style:			int
		:param labelText:		Text for label to left of text field
		:type labelText:		str
		:param buttonText:		Text for button which launches the file dialog
		:type buttonText:		str
		:param toolTip:			Help text
		:type toolTip:			str
		:param dialogTitle:		Title used in file dialog
		:type dialogTitle:		str
		:param initialValue:	The initial value of the TextCtrl
		:type initialValue:		str
		:param changeCallback:	Optional callback called for all changes in value of the control
		:type changeCallback:
		:param labelWidth:		Width of the label
		:type labelWidth:
		:param name:
		:type name:
		:param show_cancel_btn:	Whether to show or hide the cancel button.
		:type show_cancel_btn:	Bool
		:param dialog_title:	The title of the FileDialog
		:type dialog_title:		str
		:param fileMask:		File mask (glob pattern, such as *.*) to use in file dialog. See wx.FileDialog for more information
		:type fileMask:			str
		"""
		
		# Store Variables
		self._parent = parent
		self.labelText = labelText
		self.buttonText = buttonText
		self.toolTip = toolTip
		self.dialogTitle = dialogTitle
		self.initialValue = initialValue
		self.fileMask = fileMask
		self.dialog_title = dialog_title
		self.file_dialog_kwargs = kwargs
		self.style = style
		self.changeCallback = changeCallback
		self.callCallback = True
		self.labelWidth = labelWidth
		self.show_cancel_btn = show_cancel_btn
		
		# create the dialog
		self.createDialog(parent, id, pos, size, style, name)
		# Setting a value causes the changeCallback to be called.
		# In this case that would be before the return of the
		# constructor. Not good. So a default value on
		# SetValue is used to disable the callback
		self.SetValue(initialValue, 0)
	
	# self.SetMinSize((-1, 34))
	# # if size.y < 34:
	# # 	size.y = 34
	# self.SetSize(size)
	
	def createTextControl(self):
		"""Create the text control"""
		# textControl = wx.TextCtrl(self, -1)
		# textControl = SearchCtrl(self, -1, style=wx.BORDER_NONE)# TODO: make this work, style=self.style)
		textControl = ClearableTextCtrl(self, -1)
		# textControl.ShowSearchButton(False)
		# textControl.SetDescriptiveText("")
		# if self.show_cancel_btn:
		# 	textControl.ShowCancelButton(self.show_cancel_btn)
		if self.toolTip:
			textControl.SetToolTip(self.toolTip)
		if self.changeCallback:
			textControl.Bind(wx.EVT_TEXT, self.OnChanged)
			textControl.Bind(wx.EVT_COMBOBOX, self.OnChanged)
		textControl.SetMinSize((-1, 29))
		# textControl.SetSize((-1, 29))
		
		return textControl
	
	def createBrowseButton(self):
		"""Create the browse-button control"""
		# button = wx.Button(self, -1, self.buttonText)
		button = wx.BitmapButton(self, -1,
								 bitmap=wx.ArtProvider.GetBitmap(wx.ART_FOLDER, wx.ART_OTHER, wx.Size(16, 16)))
		# button.SetToolTip(self.toolTip)
		button.SetToolTip(self.buttonText)
		button.Bind(wx.EVT_BUTTON, self.OnBrowse)
		# button.SetMinSize((29, 29))
		# button.SetSize((29, 29))
		return button
	
	def OnBrowse(self, event=None):
		"""Going to browse for file..."""
		
		if self.GetValue() == '':
			default_path = pathlib.Path(self.initialValue).parent
		else:
			default_path = pathlib.Path(self.GetValue())
			if default_path.is_file():
				default_path = default_path.parent
		
		if not default_path.exists():
			default_path = pathlib.Path.home()
		
		default_path = str(default_path)
		
		pathname = file_dialog_wildcard(
				self, wildcard=self.fileMask,
				style=self.style,
				defaultDir=default_path,
				title=self.dialog_title,
				)
		
		if pathname:
			self.textControl.ChangeValue(pathname[0])
			self.textControl.SetFocus()
	
	def GetValue(self):
		"""
		Gets the contents of the control.

		Notice that for a multiline text control, the lines will be separated by (Unix-style) \n characters, even under Windows where they are separated by a \r\n sequence in the native control.

		:rtype:	string
		"""
		
		return self.textControl.GetValue()
	
	def SetValue(self, value, callBack=1):
		"""
		Sets the new text control value.

		It also marks the control as not-modified which means that IsModified() would return False immediately after the call to SetValue .

		The insertion point is set to the start of the control (i.e. position 0) by this function unless the control value doesn’t change at all, in which case the insertion point is left at its original position.

		Note that, unlike most other functions changing the controls values, this function generates a wxEVT_TEXT event. To avoid this you can use ChangeValue instead.

		Parameters:	value (string) – The new value to set. It may contain newline characters if the text control is multi-line.
		"""
		
		save = self.callCallback
		self.callCallback = callBack
		self.textControl.SetValue(value)
		self.textControl.SetForegroundColour(wx.BLACK)
		self.callCallback = save
	
	def GetLabel(self):
		""" Retrieve the label's current text """
		return self.label.GetLabel()
	
	def SetLabel(self, value):
		""" Set the label's current text """
		rvalue = self.label.SetLabel(value)
		self.Refresh(True)
		return rvalue
	
	def GetLineLength(self, lineNo):
		"""
		Gets the length of the specified line, not including any trailing newline character(s).

		:param lineNo: Line number (starting from zero).
		:type lineNo: int

		:return: The length of the line, or -1 if lineNo was invalid.
		:rtype: int
		"""
		
		return self.textControl.GetLineLength(lineNo)
	
	def GetLineText(self, lineNo):
		"""
		Returns the contents of a given line in the text control, not including any trailing newline character(s).

		:param lineNo: Line number (starting from zero).
		:type lineNo: int

		:return: The contents of the line.
		:rtype: string
		"""
		
		return self.textControl.GetLineText(lineNo)
	
	def GetNumberOfLines(self):
		"""
		Returns the number of lines in the text control buffer.

		:return:
		:rtype: int
		"""
		
		return 1
	
	def IsModified(self):
		"""
		Returns True if the text has been modified by user.

		Note that calling SetValue doesn’t make the control modified.

		:return:
		:rtype: bool
		"""
		
		return self.textControl.IsModified()
	
	def IsMultiLine(self):
		"""
		Returns True if this is a multi line edit control and False otherwise.

		:return:
		:rtype: bool
		"""
		
		return False
	
	def IsSingleLine(self):
		"""
		Returns True if this is a single line edit control and False otherwise.

		:return:
		:rtype: bool
		"""
		
		return True
	
	def MarkDirty(self):
		"""
		Mark text as modified (dirty).

		:return:
		:rtype:
		"""
		
		return self.textControl.MarkDirty()
	
	def SetModified(self, modified):
		"""
		Marks the control as being modified by the user or not.

		:param modified:
		:type modified: bool

		:return:
		:rtype:
		"""
		
		return self.textControl.SetModified(modified)
	
	def AppendText(self, text):
		"""
		Appends the text to the end of the text control.

		:param text: Text to write to the text control
		:type text: string
		"""
		return self.textControl.AppendText(text)
	
	#
	# def AutoCompleteDirectories(self):
	# 	"""
	# 	Call this function to enable auto-completion of the text using the file
	# 	system directories.
	#
	# 	Unlike AutoCompleteFileNames which completes both file names and
	# 	directories, this function only completes the directory names.
	#
	# 	Notice that currently this function is only implemented in wxMSW port
	# 	and does nothing under the other platforms.
	#
	# 	:return: True if the auto-completion was enabled or False if the
	# 	operation failed, typically because auto-completion is not supported
	# 	by the current platform.
	# 	:rtype: bool
	# 	"""
	#
	# 	return self.textControl.AutoCompleteDirectories()
	#
	# def AutoCompleteFileNames(self):
	# 	"""
	# 	Call this function to enable auto-completion of the text typed in a
	# 	single-line text control using all valid file system paths.
	#
	# 	Notice that currently this function is only implemented in wxMSW port
	# 	and does nothing under the other platforms.
	#
	# 	:return: True if the auto-completion was enabled or False if the
	# 	operation failed, typically because auto-completion is not supported
	# 	by the current platform.
	# 	:rtype: bool
	# 	"""
	#
	# 	return self.textControl.AutoCompleteFileNames()
	#
	#
	def CanCopy(self):
		"""
		Returns True if the selection can be copied to the clipboard.

		:rtype: bool
		"""
		
		return self.textControl.CanCopy()
	
	def CanCut(self):
		"""
		Returns True if the selection can be cut to the clipboard.

		:rtype: bool
		"""
		
		return self.textControl.CanCut()
	
	def CanPaste(self):
		"""
		Returns True if the contents of the clipboard can be pasted into the text control.

		On some platforms (Motif, GTK) this is an approximation and returns True if the control is editable, False otherwise.

		:rtype: bool
		"""
		
		return self.textControl.CanPaste()
	
	def CanRedo(self):
		"""
		Returns True if there is a redo facility available and the last operation can be redone.

		:rtype: bool
		"""
		
		return self.textControl.CanRedo()
	
	def CanUndo(self):
		"""
		Returns True if there is an undo facility available and the last operation can be undone.

		:rtype:	bool
		"""
		
		return self.textControl.CanUndo()
	
	def ChangeValue(self, value):
		"""
		Sets the new text control value.

		It also marks the control as not-modified which means that IsModified()
		would return False immediately after the call to ChangeValue .

		The insertion point is set to the start of the control (i.e. position 0) by this function.

		This functions does not generate the wxEVT_TEXT event but otherwise is identical to SetValue .

		:param value: The new value to set. It may contain newline characters if the text control is multi-line.
		:type value: str

		:return:
		:rtype:
		"""
		
		return self.textControl.ChangeValue(value)
	
	def Clear(self):
		"""
		Clears the text in the control.

		Note that this function will generate a wxEVT_TEXT event, i.e. its effect is identical to calling SetValue (“”).
		"""
		self.textControl.Clear()
	
	def Copy(self):
		"""
		Copies the selected text to the clipboard.
		"""
		
		return self.textControl.Copy()
	
	def Cut(self):
		"""
		Copies the selected text to the clipboard and removes it from the control.
		"""
		
		return self.textControl.Cut()
	
	def GetLastPosition(self):
		"""
		Returns the zero based index of the last position in the text control, which is equal to the number of characters in the control.

		:rtype:	wx.TextPos
		"""
		
		return self.textControl.GetLastPosition()
	
	def GetRange(self, from_, to_):
		"""
		Returns the string containing the text starting in the positions
		from and up to to in the control.

		The positions must have been returned by another wx.TextCtrl method.

		:param from_:
		:type from_: int
		:param to_:
		:type to_: int

		:return:
		:rtype: str
		"""
		
		return self.textControl.GetRange(from_, to_)
	
	def GetSelection(self):
		"""
		Gets the current selection span.

		If the returned values are equal, there was no selection. Please note
		that the indices returned may be used with the other wx.TextCtrl methods
		but don’t necessarily represent the correct indices into the string
		returned by GetValue.

		:return:
		:rtype: tuple
		"""
		
		return self.textControl.GetSelection()
	
	def GetStringSelection(self):
		"""
		Gets the text currently selected in the control.

		If there is no selection, the returned string is empty.

		:return:
		:rtype: string
		"""
		
		return self.textControl.GetStringSelection()
	
	def IsEditable(self):
		"""
		Returns True if the controls contents may be edited by user (note that it always can be changed by the program).

		In other words, this functions returns True if the control hasn’t been put in read-only mode by a previous call to SetEditable .

		:rtype:	bool
		"""
		
		return True
	
	def IsEmpty(self):
		"""
		Returns True if the control is currently empty.

		This is the same as GetValue .empty() but can be much more efficient for the multiline controls containing big amounts of text.

		:rtype:	bool
		"""
		
		return self.textControl.IsEmpty()
	
	def Paste(self):
		"""
		Pastes text from the clipboard to the text item.
		"""
		
		return self.textControl.Paste()
	
	def Redo(self):
		"""
		If there is a redo facility and the last operation can be redone, redoes the last operation.

		Does nothing if there is no redo facility.
		"""
		
		return self.textControl.Redo()
	
	def Remove(self, from_, to_):
		"""
		Removes the text starting at the first given position up to (but not including) the character at the last position.

		This function puts the current insertion point position at to as a side effect.

		:param from_: The first position
		:type from_: int
		:param to_: The last position
		:type to_: int
		"""
		
		return self.textControl.Remove(from_, to_)
	
	def Replace(self, from_, to_, value):
		"""
		Replaces the text starting at the first position up to (but not including) the character at the last position with the given text.

		This function puts the current insertion point position at to as a side effect.

		:param from_: The first position
		:type from_: int
		:param to_: The last position
		:type to_: int
		:param value: The value to replace the existing text with
		:type value: string
		"""
		
		return self.textControl.Replace(from_, to_, value)
	
	def SelectAll(self):
		"""
		Selects all text in the control.

		See also SetSelection
		"""
		
		return self.textControl.SelectAll()
	
	def SelectNone(self):
		"""
		Deselects selected text in the control.
		"""
		
		return self.textControl.SelectNone()
	
	def SetSelection(self, from_, to_):
		"""
		Selects the text starting at the first position up to (but not including) the character at the last position.

		If both parameters are equal to -1 all text in the control is selected.

		Notice that the insertion point will be moved to from by this function.

		:param from_: The first position
		:type from_: long
		:param to_: The last position
		:type to_: long

		See also SelectAll
		"""
		
		return self.textControl.SetSelection(from_, to_)
	
	def Undo(self):
		"""
		If there is an undo facility and the last operation can be undone, undoes the last operation.

		Does nothing if there is no undo facility.
		"""
		
		return self.textControl.Undo()
	
	def WriteText(self, text):
		"""
		Writes the text into the text control at the current insertion position.

		:param text: Text to write to the text control
		:type text: string
		"""
		
		return self.textControl.WriteText(text)


class FileBrowseCtrlWithHistory(FileBrowseCtrl):
	"""
	with following additions:
		__init__(..., history=None)

			history -- optional list of paths for initial history drop-down
				(must be passed by name, not a positional argument)
				If history is callable it will must return a list used
				for the history drop-down

			changeCallback -- as for FileBrowseCtrl, but with a work-around
				for win32 systems which don't appear to create wx.EVT_COMBOBOX
				events properly.  There is a (slight) chance that this work-around
				will cause some systems to create two events for each Combobox
				selection. If you discover this condition, please report it!

			As for a FileBrowseCtrl.__init__ otherwise.

		GetHistoryControl()
			Return reference to the control which implements interfaces
			required for manipulating the history list.  See GetHistoryControl
			documentation for description of what that interface is.

		GetHistory()
			Return current history list

		SetHistory( value=(), selectionIndex = None )
			Set current history list, if selectionIndex is not None, select that index

		"""
	
	def __init__(self, *arguments, **namedarguments):
		self.history = namedarguments.get("history")
		if self.history:
			del namedarguments["history"]
		
		self.historyCallBack = None
		if callable(self.history):
			self.historyCallBack = self.history
			self.history = None
		name = namedarguments.get('name', 'fileBrowseButtonWithHistory')
		namedarguments['name'] = name
		FileBrowseCtrl.__init__(self, *arguments, **namedarguments)
	
	def createTextControl(self):
		"""Create the text control"""
		textControl = wx.ComboBox(self, -1, style=wx.CB_DROPDOWN)
		textControl.SetToolTip(self.toolTip)
		textControl.Bind(wx.EVT_SET_FOCUS, self.OnSetFocus)
		if self.changeCallback:
			textControl.Bind(wx.EVT_TEXT, self.OnChanged)
			textControl.Bind(wx.EVT_COMBOBOX, self.OnChanged)
		if self.history:
			history = self.history
			self.history = None
			self.SetHistory(history, control=textControl)
		return textControl
	
	def GetHistoryControl(self):
		"""
		Return a pointer to the control which provides (at least)
		the following methods for manipulating the history list:

			Append( item ) -- add item
			Clear() -- clear all items
			Delete( index ) -- 0-based index to delete from list
			SetSelection( index ) -- 0-based index to select in list

		Semantics of the methods follow those for the wxComboBox control
		"""
		return self.textControl
	
	def SetHistory(self, value=(), selectionIndex=None, control=None):
		"""Set the current history list"""
		if control is None:
			control = self.GetHistoryControl()
		if self.history == value:
			return
		self.history = value
		# Clear history values not the selected one.
		tempValue = control.GetValue()
		# clear previous values
		control.Clear()
		control.SetValue(tempValue)
		# walk through, appending new values
		for path in value:
			control.Append(path)
		if selectionIndex is not None:
			control.SetSelection(selectionIndex)
	
	def GetHistory(self):
		"""Return the current history list"""
		if self.historyCallBack != None:
			return self.historyCallBack()
		elif self.history:
			return list(self.history)
		else:
			return []
	
	def OnSetFocus(self, event):
		"""When the history scroll is selected, update the history"""
		if self.historyCallBack != None:
			self.SetHistory(self.historyCallBack(), control=self.textControl)
		event.Skip()
	
	if wx.Platform == "__WXMSW__":
		def SetValue(self, value, callBack=1):
			""" Convenient setting of text control value, works
				around limitation of wx.ComboBox """
			save = self.callCallback
			self.callCallback = callBack
			self.textControl.SetValue(value)
			self.callCallback = save
			
			# Hack to call an event handler
			class LocalEvent:
				def __init__(self, string):
					self._string = string
				
				def GetString(self):
					return self._string
			
			if callBack == 1:
				# The callback wasn't being called when SetValue was used ??
				# So added this explicit call to it
				self.changeCallback(LocalEvent(value))


class DirBrowseCtrl(FileBrowseCtrl):
	def __init__(
			self, parent, id=-1, pos=wx.DefaultPosition, size=wx.DefaultSize,
			style=wx.TAB_TRAVERSAL | wx.DD_DEFAULT_STYLE | wx.DD_NEW_DIR_BUTTON, labelText='Select a directory:',
			buttonText='Browse', toolTip='Type directory name or browse to select',
			dialogTitle='', initialValue=None, changeCallback=None,
			name='DirBrowseCtrl'
			):
		
		if initialValue is None:
			initialValue = os.getcwd()
		
		FileBrowseCtrl.__init__(
				self, parent, id, pos, size, style, labelText, buttonText,
				toolTip, dialogTitle, initialValue,
				changeCallback=changeCallback, name=name)
	
	def OnBrowse(self, ev=None):
		
		if self.GetValue() == '':
			default_path = pathlib.Path(self.initialValue)
		else:
			default_path = pathlib.Path(self.GetValue())
			if default_path.is_file():
				default_path = default_path.parent
		
		if not default_path.exists():
			default_path = pathlib.Path.home()
		
		default_path = str(default_path)
		
		dialog = wx.DirDialog(
				self, message=self.dialogTitle,
				defaultPath=default_path, style=self.style)
		
		if dialog.ShowModal() == wx.ID_OK:
			self.ChangeValue(dialog.GetPath())
		dialog.Destroy()
		
		self.textControl.SetFocus()




#  !/usr/bin/env python
#   -*- coding: utf-8 -*-
#
#  border_config.py
"""
Dialog for configuring borders for charts
"""
#
#  Copyright (c) 2019-2020 Dominic Davis-Foster <dominic@davis-foster.co.uk>
#
#  This file was originally part of GunShotMatch
#  Copyright (c) 2019 Dominic Davis-Foster <dominic@davis-foster.co.uk>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#

# 3rd party
import wx

# begin wxGlade: dependencies
# end wxGlade

# begin wxGlade: extracode
# end wxGlade


class border_config(wx.Dialog):
	def __init__(self, parent, chromatogram_figure, *args, **kwds):
		self.chromatogram_figure = chromatogram_figure
		args = (parent,) + args
		# begin wxGlade: border_config.__init__
		kwds["style"] = kwds.get("style", 0) | wx.DEFAULT_DIALOG_STYLE
		wx.Dialog.__init__(self, *args, **kwds)
		self.main_panel = wx.Panel(self, wx.ID_ANY)
		self.top_border_value = wx.SpinCtrlDouble(self.main_panel, wx.ID_ANY, "0.9", min=0.0, max=2.0)
		self.top_border_value.SetDigits(3)
		self.bottom_border_value = wx.SpinCtrlDouble(self.main_panel, wx.ID_ANY, "0.125", min=0.0, max=2.0)
		self.bottom_border_value.SetDigits(3)
		self.left_border_value = wx.SpinCtrlDouble(self.main_panel, wx.ID_ANY, "0.1", min=0.0, max=2.0)
		self.left_border_value.SetDigits(3)
		self.right_border_value = wx.SpinCtrlDouble(self.main_panel, wx.ID_ANY, "0.97", min=0.0, max=2.0)
		self.right_border_value.SetDigits(3)
		self.tight_layout_button = wx.Button(self.main_panel, wx.ID_ANY, "Tight Layout")
		self.close_btn = wx.Button(self.main_panel, wx.ID_ANY, "Close")

		self.__set_properties()
		self.__do_layout()

		self.Bind(wx.EVT_SPINCTRLDOUBLE, self.update_borders, self.top_border_value)
		self.Bind(wx.EVT_TEXT, self.update_borders, self.top_border_value)
		self.Bind(wx.EVT_TEXT_ENTER, self.update_borders, self.top_border_value)
		self.Bind(wx.EVT_SPINCTRLDOUBLE, self.update_borders, self.bottom_border_value)
		self.Bind(wx.EVT_TEXT, self.update_borders, self.bottom_border_value)
		self.Bind(wx.EVT_TEXT_ENTER, self.update_borders, self.bottom_border_value)
		self.Bind(wx.EVT_SPINCTRLDOUBLE, self.update_borders, self.left_border_value)
		self.Bind(wx.EVT_TEXT, self.update_borders, self.left_border_value)
		self.Bind(wx.EVT_TEXT_ENTER, self.update_borders, self.left_border_value)
		self.Bind(wx.EVT_SPINCTRLDOUBLE, self.update_borders, self.right_border_value)
		self.Bind(wx.EVT_TEXT, self.update_borders, self.right_border_value)
		self.Bind(wx.EVT_TEXT_ENTER, self.update_borders, self.right_border_value)
		self.Bind(wx.EVT_BUTTON, self.apply_tight_layout, self.tight_layout_button)
		self.Bind(wx.EVT_BUTTON, self.close_dialog, self.close_btn)
		# end wxGlade

	def __set_properties(self):
		# begin wxGlade: border_config.__set_properties
		self.SetTitle("Configure Borders")
		self.top_border_value.SetMinSize((120, -1))
		self.top_border_value.SetIncrement(0.005)
		self.bottom_border_value.SetMinSize((120, -1))
		self.bottom_border_value.SetIncrement(0.005)
		self.left_border_value.SetMinSize((120, -1))
		self.left_border_value.SetIncrement(0.005)
		self.right_border_value.SetMinSize((120, -1))
		self.right_border_value.SetIncrement(0.005)
		self.tight_layout_button.SetMinSize((120, -1))
		# end wxGlade

	def __do_layout(self):
		# begin wxGlade: border_config.__do_layout
		parent_sizer = wx.BoxSizer(wx.VERTICAL)
		main_sizer = wx.BoxSizer(wx.VERTICAL)
		spinner_grid = wx.FlexGridSizer(4, 2, 4, 7)
		borders_label = wx.StaticText(self.main_panel, wx.ID_ANY, "Configure Borders")
		borders_label.SetFont(wx.Font(12, wx.FONTFAMILY_DEFAULT, wx.FONTSTYLE_NORMAL, wx.FONTWEIGHT_BOLD, 0, ""))
		main_sizer.Add(borders_label, 0, wx.BOTTOM, 7)
		top_border_label = wx.StaticText(self.main_panel, wx.ID_ANY, "Top: ")
		spinner_grid.Add(top_border_label, 0, wx.ALIGN_CENTER_VERTICAL, 0)
		spinner_grid.Add(self.top_border_value, 0, wx.ALIGN_CENTER_VERTICAL, 0)
		bottom_border_label = wx.StaticText(self.main_panel, wx.ID_ANY, "Bottom: ")
		spinner_grid.Add(bottom_border_label, 0, wx.ALIGN_CENTER_VERTICAL, 0)
		spinner_grid.Add(self.bottom_border_value, 0, wx.ALIGN_CENTER_VERTICAL, 0)
		left_border_label = wx.StaticText(self.main_panel, wx.ID_ANY, "Left: ")
		spinner_grid.Add(left_border_label, 0, wx.ALIGN_CENTER_VERTICAL, 0)
		spinner_grid.Add(self.left_border_value, 0, wx.ALIGN_CENTER_VERTICAL, 0)
		right_border_label = wx.StaticText(self.main_panel, wx.ID_ANY, "Right: ")
		spinner_grid.Add(right_border_label, 0, wx.ALIGN_CENTER_VERTICAL, 0)
		spinner_grid.Add(self.right_border_value, 0, wx.ALIGN_CENTER_VERTICAL, 0)
		main_sizer.Add(spinner_grid, 0, 0, 0)
		main_sizer.Add(self.tight_layout_button, 0, wx.ALIGN_CENTER | wx.TOP, 5)
		static_line_9 = wx.StaticLine(self.main_panel, wx.ID_ANY)
		main_sizer.Add(static_line_9, 0, wx.BOTTOM | wx.EXPAND | wx.TOP, 5)
		main_sizer.Add(self.close_btn, 0, wx.ALIGN_CENTER_VERTICAL | wx.ALIGN_RIGHT, 5)
		self.main_panel.SetSizer(main_sizer)
		parent_sizer.Add(self.main_panel, 1, wx.ALL | wx.EXPAND, 10)
		self.SetSizer(parent_sizer)
		parent_sizer.Fit(self)
		self.Layout()
		# end wxGlade
	
	def update_borders(self, event):  # wxGlade: border_config.<event_handler>
		self.chromatogram_figure.subplots_adjust(
			self.left_border_value.GetValue(),
			self.bottom_border_value.GetValue(),
			self.right_border_value.GetValue(),
			self.top_border_value.GetValue()
		)
		self.chromatogram_figure.canvas.draw_idle()
	
	def apply_tight_layout(self, event):  # wxGlade: border_config.<event_handler>
		self.chromatogram_figure.tight_layout()
		self.chromatogram_figure.canvas.draw_idle()
		
		# set SpinCtrls to new values
		self.right_border_value.SetValue(self.chromatogram_figure.subplotpars.right)
		self.left_border_value.SetValue(self.chromatogram_figure.subplotpars.left)
		self.bottom_border_value.SetValue(self.chromatogram_figure.subplotpars.bottom)
		self.top_border_value.SetValue(self.chromatogram_figure.subplotpars.top)
	
	def close_dialog(self, event):  # wxGlade: border_config.<event_handler>
		self.Destroy()

# end of class border_config



#!/usr/bin/env python
#   -*- coding: utf-8 -*-
#
#  ClearableTextCtrl.py
"""
A TextCtrl with a button to clear its contents
"""
#
#  Copyright (c) 2020 Dominic Davis-Foster <dominic@davis-foster.co.uk>
#
#  Adapted from src/generic/srchctlg.cpp (part of wxWidgets, https://github.com/wxWidgets/wxWidgets)
#  Copyright (c) 2006 Vince Harron.
#  Licenced under the wxWindows licence
#
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#

# 3rd party
import wx
from wx.lib.embeddedimage import PyEmbeddedImage

# the margin between the text control and the clear button
MARGIN = 3

# arguments to wxColour::ChangeLightness() for making the search/clear
# bitmaps foreground colour, respectively
CANCEL_BITMAP_LIGHTNESS = 16  # a bit more lighter


ClearableTextCtrlNameStr = "ClearableTextCtrl"


class CTCWidget(wx.TextCtrl):
	"""
	CTCWidget: text control used by ClearableTextCtrl
	"""
	
	def __init__(self, parent, value, style, validator):
		"""
		:param parent: The parent window.
		:type parent: ClearableTextCtrl
		:param value: The initial value of the text control
		:type value: str
		:param style: The style of the text control
		:type style: int
		"""
		
		wx.TextCtrl.__init__(self, parent, value=value, style=style, validator=validator)
		self.parent = parent
		
		# Ensure that our best size is recomputed using our overridden DoGetBestSize()
		self.InvalidateBestSize()
		
		self.Bind(wx.EVT_TEXT, self.OnText, id=wx.ID_ANY)
		self.Bind(wx.EVT_TEXT_ENTER, self.OnTextEnter, id=wx.ID_ANY)
		self.Bind(wx.EVT_TEXT_MAXLEN, self.OnText, id=wx.ID_ANY)
		
	def GetMainWindowOfCompositeControl(self):
		return self.parent
	
	def OnText(self, event):
		"""
		Event handler for text being entered in the control
		"""
		
		event = wx.CommandEvent(event)
		event.SetEventObject(self.parent)
		event.SetId(self.parent.GetId())
		
		self.parent.GetEventHandler().ProcessEvent(event)
	
	def OnTextEnter(self, event):
		"""
		Event handler for the enter/return key being pressed
		"""
		
		if not self.IsEmpty():
			event = wx.CommandEvent(wx.EVT_SEARCHCTRL_SEARCH_BTN, self.parent.GetId)
			event.SetEventObject(self.parent)
			event.SetString(self.parent.GetValue())
			
			self.parent.ProcessWindowEvent(event)
	
	"""
	
#ifdef __WXMSW__
	# We increase the text control height to be the same as for the controls
	# with border as this is what we actually need here because even though
	# this control itself is borderless, it's inside wxClearableTextCtrl which does
	# have the border and so should have the same height as the normal text
	# entries with border.
	#
	# This is a bit ugly and it would arguably be better to use whatever size
	# the base class version return and just centre the text vertically in
	# the search control but I failed to modify the code in LayoutControls()
	# to do this easily and as there is much in that code I don't understand
	# (notably what is the logic for buttons sizing?) I prefer to not touch it
	# at all.
	virtual wxSize DoGetBestSize() const wxOVERRIDE
	{
		const long flags = GetWindowStyleFlag()
		wxCTCWidget* const self = const_cast<wxCTCWidget*>(this)

		self.SetWindowStyleFlag((flags & ~wxBORDER_MASK) | wxBORDER_DEFAULT)
		wxSize size = wxTextCtrl::DoGetBestSize()

		# The calculation for no external borders in wxTextCtrl::DoGetSizeFromTextSize also
		# removes any padding around the value, which is wrong for this situation. So we
		# can't use wxBORDER_NONE to calculate a good height, in which case we just have to
		# assume a border in the code above and then subtract the space that would be taken up
		# by a themed border (the thin blue border and the white internal border).
		# Don't use FromDIP(4), this seems not needed.
		size.y -= 4

		self.SetWindowStyleFlag(flags)

		return size
	}
#endif # __WXMSW__"""

#b"                       ",

clear_btn = [
	b"25 19 2 1",
	b"   c None",
	b"+  c #000000",
	b"                         ",
	b"                         ",
	b"                         ",
	b"                         ",
	b"                         ",
	b"                         ",
	b"         +++++++++++++   ",
	b"        ++           +   ",
	b"       ++    +   +   +   ",
	b"      ++      + +    +   ",
	b"     ++        +     +   ",
	b"      ++      + +    +   ",
	b"       ++    +   +   +   ",
	b"        ++           +   ",
	b"         +++++++++++++   ",
	b"                         ",
	b"                         ",
	b"                         ",
	b"                         ",
	b"                         ",
		]


class ClearButton(wx.Control):
	"""
	Clear button for the ClearableTextCtrl
	"""
	def __init__(self, parent, eventType, bmp):
		"""
		:param parent: The parent window.
		:type parent: ClearableTextCtrl
		:param eventType:
		:type eventType:wx.PyEventBinder
		:param bmp:
		:type bmp: wx.Bitmap
		"""
		
		wx.Control.__init__(self, parent, id=wx.ID_ANY, style=wx.NO_BORDER)
		self.parent = parent
		self.m_eventType = eventType.typeId
		self.m_bmp = bmp
		
		self.SetBackgroundStyle(wx.BG_STYLE_PAINT)
		
		self.Bind(wx.EVT_LEFT_UP, self.OnLeftUp)
		self.Bind(wx.EVT_PAINT, self.OnPaint)
		
	def SetBitmapLabel(self, label):
		"""
		Set bitmap for the button
		
		:param label: Bitmap to set for the button
		:type label: wx.Bitmap
		"""
		
		self.m_bmp = label
		self.InvalidateBestSize()
	
	# The buttons in ClearableTextCtrl shouldn't accept focus from keyboard because
	# this would interfere with the usual TAB processing: the user expects
	# that pressing TAB in the search control should switch focus to the next
	# control and not give it to the button inside the same control.
	def AcceptsFocusFromKeyboard(self):
		return False
	
	def GetMainWindowOfCompositeControl(self):
		return self.parent
	
	def GetBestSize(self):
		return wx.Size(self.m_bmp.GetWidth(), self.m_bmp.GetHeight())
	
	def OnLeftUp(self, event):
		"""
		Event Handler for left mouse button being released
		"""
		
		event = wx.CommandEvent(self.m_eventType, self.parent.GetId())
		event.SetEventObject(self.parent)
		
		if self.m_eventType == wx.EVT_SEARCHCTRL_SEARCH_BTN.typeId:
			# it's convenient to have the string to search for directly in the
			# event instead of having to retrieve it from the control in the
			# event handler code later, so provide it here
			event.SetString(self.parent.GetValue())
		
		self.GetEventHandler().ProcessEvent(event)
		
		self.parent.SetFocus()
		
	def OnPaint(self, event):
		"""
		Event Handler for widget being painted
		"""
		
		dc = wx.PaintDC(self)
		
		# Clear the background in case of a user bitmap with alpha channel
		dc.GetBrush().SetColour(self.parent.GetBackgroundColour())
		#dc.SetBrush(self.parent.GetBackgroundColour())
		dc.Clear()
		
		# Draw the bitmap
		dc.DrawBitmap(self.m_bmp, 0, 0, True)
		

class ClearableTextCtrl(wx.Panel):
	"""
	TextCtrl with button to clear its contents
	"""
	
	def __init__(
			self, parent, id=wx.ID_ANY, value="", pos=wx.DefaultPosition,
			size=wx.DefaultSize, style=0, validator=wx.DefaultValidator,
			name=ClearableTextCtrlNameStr
			):
		"""
		:param parent: The parent window.
		:type parent: wx.Window
		:param id: An identifier for the control. wx.ID_ANY is taken to mean a default.
		:type id: wx.WindowID, optional
		:param value: Default text value
		:type value: str
		:param pos: The control position. The value wx.DefaultPosition indicates a default position, chosen by either the windowing system or wxWidgets, depending on platform.
		:type pos: wx.Point, optional
		:param size: The control size. The value wx.DefaultSize indicates a default size, chosen by either the windowing system or wxWidgets, depending on platform.
		:type size: wx.Size, optional
		:param style: The window style. See wx.TextCtrl.
		:type style: int, optional
		:param validator: Window validator
		:type validator: wx.Validator, optional
		:param name: Window name.
		:type name: str, optional
		"""
		
		wx.Panel.__init__(self, parent, id, pos, size, style | wx.BORDER_SUNKEN, name)
		
		textctrl_style = style | wx.TAB_TRAVERSAL | wx.BORDER_NONE | wx.TE_RICH2
		
		if wx.Platform == "__WXGTK__":
			print(122)
			# wx.BORDER_NONE doesn't work unless its a multiline textctrl
			textctrl_style |= wx.TE_MULTILINE | wx.TE_DONTWRAP
		
		self.m_text = CTCWidget(self, value, textctrl_style, validator=validator)
		self.m_clearBitmap = self.default_clear_bitmap
		self.m_clearButton = ClearButton(self, wx.EVT_SEARCHCTRL_CANCEL_BTN, self.m_clearBitmap)
		self.m_clearButton.SetSize((16, 16))
		self.LayoutControls()
		
		self.SetBackgroundColour(self.m_text.GetBackgroundColour())
		# self.m_text.SetBackgroundColour(wx.Colour())
		
		self.SetInitialSize(size)
		self.SetMinSize((-1, 29))
		self.Move(pos)

		self.Bind(wx.EVT_SEARCHCTRL_CANCEL_BTN, self.OnClearButton, id=wx.ID_ANY)
		self.Bind(wx.EVT_SIZE, self.OnSize)
		
	# def Destroy(self):
	# 	self.m_text.Destroy()
	# 	self.m_clearButton.Destroy()
	# 	wx.Control.Destroy(self)
	
	@property
	def default_clear_bitmap(self):
		"""
		Returns the default clear button bitmap for the control

		:rtype: wx.Bitmap
		"""
		
		#return Clear_Button_16.GetBitmap()
		return wx.Bitmap(clear_btn)
	
	def SetFont(self, font):
		"""
		Sets the font for the control
		
		:param font: Font to associate with this control, pass wx.NullFont to reset to the default font.
		:type font: wx.Font
		
		:return: True if the operation completes successfully, False otherwise
		:rtype: bool
		"""
		
		if not self.m_text.SetFont(font):
			return False
	
		# TODO: Recreate the bitmaps as their size may have changed.

		return True
	
	def SetBackgroundColour(self, colour):
		"""
		Sets the background colour of the control
		
		:param colour: The colour to be used as the background colour; pass wx.NullColour to reset to the default colour.
		:type colour: wx.Colour
		
		.. note::
		
			You may want to use wx.SystemSettings.GetColour to retrieve a
			suitable colour to use rather than setting an hard-coded one.
		
		:return: True if the operation completes successfully, False otherwise
		:rtype: bool
		"""
		
		if not all([
				self.m_text.SetBackgroundColour(colour),
				wx.Window.SetBackgroundColour(self, colour)
				]):
			return False
	
		# TODO: When the background changes, re-render the bitmaps so that the correct
		#  colour shows in their "transparent" area.

		return True
	
	def AppendText(self, text):
		"""
		Appends the text to the end of the text control.

		.. note::
			
			After the text is appended, the insertion point will be at the end of the text control.
			If this behaviour is not desired, the programmer should use GetInsertionPoint and SetInsertionPoint.

		:param text: Text to write to the text control.
		:type text:	str
		"""

		self.m_text.AppendText(text)
	
	def AutoComplete(self, completer):
		"""
		Enable auto-completion using the provided completer object.

		The specified completer object will be used to retrieve the list of possible completions for the already entered text and will be deleted by wx.TextEntry itself when it’s not needed any longer.
		
		:param completer: The object to be used for generating completions if not None. If it is None, auto-completion is disabled. The wx.TextEntry object takes ownership of this pointer and will delete it in any case (i.e. even if this method return False).
		:type completer: wx.TextCompleter
		
		:return: True if the auto-completion was enabled or False if the operation failed, typically because auto-completion is not supported by the current platform.
		:rtype: bool
		"""

		return self.m_text.AutoComplete(completer)
		
	def AutoCompleteDirectories(self):
		"""
		Call this function to enable auto-completion of the text using the file system directories.

		Unlike AutoCompleteFileNames which completes both file names and directories, this function only completes the directory names.
		
		Notice that currently this function is only implemented in wxMSW port and does nothing under the other platforms.
		
		:return: True if the auto-completion was enabled or False if the operation failed, typically because auto-completion is not supported by the current platform.
		:rtype: bool
		"""
	
		return self.m_text.AutoCompleteDirectories()
		
	def AutoCompleteFileNames(self):
		"""
		Call this function to enable auto-completion of the text typed in a single-line text control using all valid file system paths.

		Notice that currently this function is only implemented in wxMSW port and does nothing under the other platforms.
		
		:return: True if the auto-completion was enabled or False if the operation failed, typically because auto-completion is not supported by the current platform.
		:rtype: bool
		"""
	
		return self.m_text.AutoCompleteFileNames()
	
	def CanCopy(self):
		"""
		:return: True if the selection can be copied to the clipboard.
		:rtype:	bool
		"""
		
		return self.m_text.CanCopy()
	
	def CanCut(self):
		"""
		:return: True if the selection can be cut to the clipboard.
		:rtype:	bool
		"""
		
		return self.m_text.CanCut()
	
	def CanPaste(self):
		"""
		:return: True if the contents of the clipboard can be pasted into the text control.
		:rtype:	bool
		
		On some platforms (Motif, GTK) this is an approximation and returns True if the control is editable, False otherwise.
		"""
		
		return self.m_text.CanPaste()
	
	def CanRedo(self):
		"""
		:return: True if there is a redo facility available and the last operation can be redone.
		:rtype: bool
		"""
		
		return self.m_text.CanRedo()
	
	def CanUndo(self):
		"""
		:return: True if there is an undo facility available and the last operation can be undone.
		:rtype: bool
		"""
		
		return self.m_text.CanUndo()
	
	def ChangeValue(self, value):
		"""
		Sets the new text control value.

		It also marks the control as not-modified which means that IsModified() would return False immediately after the call to ChangeValue .
		
		The insertion point is set to the start of the control (i.e. position 0) by this function.
		
		This functions does not generate the wxEVT_TEXT event but otherwise is identical to SetValue .
		
		See User Generated Events vs Programmatically Generated Events for more information.
		
		:param value: The new value to set. It may contain newline characters if the text control is multi-line
		:type value: str
		"""
		
		self.m_text.ChangeValue(value)
	
	def Clear(self):
		"""
		Clears the text in the control.

		Note that this function will generate a wxEVT_TEXT event, i.e. its effect is identical to calling SetValue (“”).
		"""
		
		self.m_text.Clear()
	
	def Copy(self):
		"""Copies the selected text to the clipboard."""
		
		self.m_text.Copy()
	
	def Cut(self):
		"""Copies the selected text to the clipboard and removes it from the control."""
		
		self.m_text.Cut()
	
	def DiscardEdits(self):
		"""Resets the internal modified flag as if the current changes had been saved."""
		
		self.m_text.DiscardEdits()
	
	def EmulateKeyPress(self, event):
		"""
		This function inserts into the control the character which would have been inserted if the given key event had occurred in the text control.

		The event object should be the same as the one passed to EVT_KEY_DOWN handler previously by wxWidgets. Please note that this function doesn’t currently work correctly for all keys under any platform but MSW.

		:param event:
		:type event: wx.KeyEvent
		
		:return: True if the event resulted in a change to the control, False otherwise.
		:rtype: bool
		"""
		
		return self.m_text.EmulateKeyPress(event)
	
	def GetBestClientSize(self):
		"""

		:return:
		:rtype:	wx.Size
		"""
		
		size = self.m_text.GetBestSize()
		
		if self.IsClearButtonVisible():
			size.x += self.m_clearButton.GetBestSize().x + self.FromDIP(MARGIN)
		
		# horizontalBorder = FromDIP(1) + (size.y - size.y * 14 / 21 ) / 2
		horizontalBorder = (size.y - size.y * 14 / 21) / 2
		size.x += 2 * horizontalBorder
		
		return size
	
	def GetCompositeWindowParts(self):
		"""

		:return:
		:rtype:	wxWindowList
		"""
		
		return [self.m_text, self.m_clearButton]
	
	def GetDefaultStyle(self):
		"""
		:return: The style currently used for the new text.
		:rtype: wx.TextAttr
		"""
		
		return self.m_text.GetDefaultStyle()
	
	def GetInsertionPoint(self):
		"""
		Returns the insertion point, or cursor, position.

		This is defined as the zero based index of the character position to the right of the insertion point. For example, if the insertion point is at the end of the single-line text control, it is equal to GetLastPosition .

		:return:
		:rtype:	int
		"""
		
		return self.m_text.GetInsertionPoint()
	
	def GetLastPosition(self):
		"""
		Returns the zero based index of the last position in the text control, which is equal to the number of characters in the control.

		:return:
		:rtype: wx.TextPos
		"""
		return self.m_text.GetLastPosition()
	
	def GetLineLength(self, lineNo):
		"""
		Gets the length of the specified line, not including any trailing newline character(s).
		
		:param lineNo: Line number (starting from zero).
		:type lineNo: int

		:return: The length of the line, or -1 if lineNo was invalid.
		:rtype:	int
		"""
		
		return self.m_text.GetLineLength(lineNo)
	
	def GetLineText(self, lineNo):
		"""
		Returns the contents of a given line in the text control, not including any trailing newline character(s).
		
		:param lineNo: Line number (starting from zero).
		:type lineNo: int

		:return: The contents of the line.
		:rtype:	str
		"""
		
		return self.m_text.GetLineText(lineNo)
	
	# GetMargins
	
	# @staticmethod
	# def GetMultiplier():
	# 	"""
	# 	Icons are rendered at 3-8 times larger than necessary and downscaled for antialiasing
	#
	# 	:return:
	# 	:rtype: int
	# 	"""
	#
	# 	depth = wx.DisplayDepth()
	#
	# 	if depth >= 24:
	# 		return 8
	# 	else:
	# 		return 6
	
	def GetNumberOfLines(self):
		"""
		Returns the number of lines in the text control buffer.
		
		:return:
		:rtype:	int
		"""
		
		return 1
	
	def GetRange(self, from_, to_):
		"""
		Returns the string containing the text starting in the positions from and up to to in the control.

		The positions must have been returned by another wx.TextCtrl method. Please note that the positions in a multiline wx.TextCtrl do not correspond to the indices in the string returned by GetValue because of the different new line representations ( CR or CR LF) and so this method should be used to obtain the correct results instead of extracting parts of the entire value. It may also be more efficient, especially if the control contains a lot of data.

		:param from_:
		:type from_: int
		:param to_:
		:type to_: int

		:return:
		:rtype: str
		"""
		
		return self.m_text.GetRange(from_, to_)
	
	def GetSelection(self):
		"""
		Gets the current selection span.

		If the returned values are equal, there was no selection. Please note that the indices returned may be used with the other wx.TextCtrl methods but don’t necessarily represent the correct indices into the string returned by GetValue for multiline controls under Windows (at least,) you should use GetStringSelection to get the selected text.

		:return:
		:rtype: tuple
		"""

		return self.m_text.GetSelection()
	
	def GetStringSelection(self):
		"""
		Gets the text currently selected in the control.

		If there is no selection, the returned string is empty.

		:return:
		:rtype: str
		"""
		
		return self.m_text.GetStringSelection()
	
	def GetStyle(self, position, style):
		"""
		Returns the style at this position in the text control.
		
		Not all platforms support this function.
		
		:param position:
		:type position: int
		:param style:
		:type style: wx.TextAttr

		:return: True on success, False if an error occurred (this may also mean that the styles are not supported under this platform).
		:rtype: bool
		"""
		
		return self.m_text.GetStyle(position, style)
	
	def GetValue(self):
		"""
		Gets the contents of the control.
		
		Notice that for a multiline text control, the lines will be separated by (Unix-style) \n characters, even under Windows where they are separated by a \r\n sequence in the native control.
		
		:return:
		:rtype:	str
		"""
		
		return self.m_text.GetValue()
	
	def HitTestPos(self, pt):
		"""
		Finds the position of the character at the specified point.

		If the return code is not TE_HT_UNKNOWN the position of the character closest to this position is returned, otherwise the output parameter is not modified.
		
		Please note that this function is currently only implemented in Univ, wxMSW and wxGTK ports and always returns TE_HT_UNKNOWN in the other ports.

		:param pt:
		:type pt:

		:return:
		:rtype:	wxTextCtrlHitTestResult
		"""
		
		return self.m_text.HitTestPos(pt)
	
	def HitTest(self, pt):
		"""
		Finds the row and column of the character at the specified point.

		If the return code is not TE_HT_UNKNOWN the row and column of the character closest to this position are returned, otherwise the output parameters are not modified.
		
		Please note that this function is currently only implemented in Univ, wxMSW and wxGTK ports and always returns TE_HT_UNKNOWN in the other ports.

		NB: pt is in device coords (not adjusted for the client area origin nor scrolling)

		:param pt:
		:type pt:

		:return:
		:rtype:	wxTextCtrlHitTestResult
		"""
		
		return self.m_text.HitTest(pt)
	
	def IsClearButtonVisible(self):
		"""Returns the clear button’s visibility state"""
		return self.m_clearButton and self.m_clearButton.IsShown()
	
	def IsEditable(self):
		"""
		Returns True if the controls contents may be edited by user (note that it always can be changed by the program).
		
		In other words, this functions returns True if the control hasn't been put in read-only mode by a previous call to SetEditable .
		
		:return:
		:rtype:	bool
		"""
		
		return self.m_text.IsEditable()
	
	def IsEmpty(self):
		"""
		Returns True if the control is currently empty.

		This is the same as GetValue.empty() but can be much more efficient for the multiline controls containing big amounts of text.
		
		:return:
		:rtype:	bool
		"""
		
		return self.m_text.IsEmpty()
	
	def IsModified(self):
		"""
		Returns True if the text has been modified by user.
		
		Note that calling SetValue doesn’t make the control modified.
		
		:return:
		:rtype:	bool
		"""
		
		return self.m_text.IsModified()
	
	def IsMultiLine(self):
		"""
		Returns True if this is a multi line edit control and False otherwise.
		
		:return:
		:rtype:	bool
		"""
		
		return self.m_text.IsMultiLine()
	
	def IsSingleLine(self):
		"""
		Returns True if this is a single line edit control and False otherwise.
		
		:return:
		:rtype:	bool
		"""
		
		return self.m_text.IsSingleLine()
		
	def LayoutControls(self):
		if not self.m_text:
			return
		
		total_size = self.GetClientSize()
		overall_width = total_size.x
		overall_height = total_size.y
		

		
		text_size = self.m_text.GetBestSize()
		clear_size = self.m_clearButton.GetBestSize()
		clear_margin = MARGIN
		
		# make room for the clear button
		horizontalBorder = 1 + (text_size.y - text_size.y * 14 / 21) / 2
		
		x = 0
		textWidth = overall_width
		
		
		textWidth -= horizontalBorder
		
		textWidth -= + clear_size.x + clear_margin + 1
		if textWidth < 0:
			textWidth = 0
		
		# position the subcontrols inside the client area
		
		if wx.Platform == "__WXMSW__":
			# The text control is too high up on Windows; normally a text control looks OK because
			# of the white border that's part of the theme border. We can also remove a pixel from
			# the height to fit the text control in, because the padding in EDIT_HEIGHT_FROM_CHAR_HEIGHT
			# is already generous.
			textY = 1
		else:
			textY = 0
		
		self.m_text.SetSize(x, textY, textWidth, overall_height - textY - 10)
		
		text_pos = self.m_text.GetPosition()
		self.m_text.SetPosition((text_pos.x + 3, text_pos.y + 5))

		x += textWidth
		
		x += clear_margin
		self.m_clearButton.SetSize(
				x,
				(overall_height - clear_size.y) / 2,
				clear_size.x, clear_size.y
				)
	
		clear_pos = self.m_clearButton.GetPosition()
		self.m_clearButton.SetPosition((clear_pos.x, clear_pos.y))
	
	def MarkDirty(self):
		"""Mark text as modified (dirty)."""

		self.m_text.MarkDirty()
	
	def OnClearButton(self, event):
		"""
		Event handler for clear button being pressed
		
		:param event:
		:type event: wx.CommandEvent
		"""
		
		self.m_text.Clear()
		event.Skip()
	
	def OnSize(self, event):
		"""
		Event handler for the size of the control being changed
		
		:param event:
		:type event: wx.SizeEvent
		"""
		
		# pass
		self.LayoutControls()
	
	def Paste(self):
		self.m_text.Paste()
		
	def PositionToXY(self, pos):
		"""
		Converts given position to a zero-based column, line number pair.

		:param pos: Position
		:type pos: int

		:return:
		:rtype: tuple
		"""
		
		return self.m_text.PositionToXY(pos)
	
	def Redo(self):
		"""
		If there is a redo facility and the last operation can be redone, redoes the last operation.
		
		Does nothing if there is no redo facility.
		"""
		
		self.m_text.Redo()
	
	def Remove(self, from_, to_):
		"""
		Removes the text starting at the first given position up to (but not including) the character at the last position.

		This function puts the current insertion point position at to as a side effect.
		
		:param from_: The first position
		:type from_:	int
		:param to_: The last position
		:type to_:	int
		"""
		
		self.m_text.Remove(from_, to_)
	
	# def RenderClearBitmap(self, x, y):
	# 	"""
	#
	# 	:param x:
	# 	:type x:	int
	# 	:param y:
	# 	:type y: 	int
	# 	:param renderDrop:
	# 	:type renderDrop:	bool
	# 	"""
	#
	# 	bg = self.GetBackgroundColour()
	# 	fg = self.GetForegroundColour().ChangeLightness(CANCEL_BITMAP_LIGHTNESS)
	#
	# 	# begin drawing code
	# 	# image starts
	#
	# 	# total size 14x14
	# 	# force 1:1 ratio
	# 	if x > y:
	# 		# x is too big
	# 		x = y
	# 	else:
	# 		# y is too big
	# 		y = x
	#
	# 	# 14x14 circle
	# 	# cross line starts (4,4)-(10,10)
	# 	# drop (13,16)-(19,6)-(16,9)
	#
	# 	multiplier = self.GetMultiplier()
	#
	# 	multiplier = 1
	#
	# 	penWidth = multiplier * x / 14
	#
	# 	bitmap = wx.Bitmap(multiplier * x, multiplier * y)
	# 	mem = wx.MemoryDC()
	# 	mem.SelectObject(bitmap)
	#
	# 	# clear background
	# 	mem.SetBrush(wx.Brush(bg))
	# 	mem.SetPen(wx.Pen(bg))
	# 	mem.DrawRectangle(0, 0, bitmap.GetWidth(), bitmap.GetHeight())
	#
	# 	# draw drop glass
	# 	mem.SetBrush(wx.Brush(fg))
	# 	mem.SetPen(wx.Pen(fg))
	# 	radius = multiplier * x / 2
	# 	mem.DrawCircle(radius, radius, radius)
	#
	# 	# draw cross
	# 	lineStartBase = 4 * x / 14
	# 	lineLength = x - 2 * lineStartBase
	#
	# 	mem.SetPen(wx.Pen(bg))
	# 	mem.SetBrush(wx.Brush(bg))
	# 	handleCornerShift = penWidth / 2
	# 	handleCornerShift = max(handleCornerShift, 1)
	# 	handlePolygon = [
	# 			wx.Point(-handleCornerShift, +handleCornerShift),
	# 			wx.Point(+handleCornerShift, -handleCornerShift),
	# 			wx.Point(multiplier * lineLength + handleCornerShift, multiplier * lineLength - handleCornerShift),
	# 			wx.Point(multiplier * lineLength - handleCornerShift, multiplier * lineLength + handleCornerShift),
	# 			]
	# 	mem.DrawPolygon(handlePolygon, multiplier * lineStartBase, multiplier * lineStartBase)
	#
	# 	handlePolygon2 = [
	# 			wx.Point(+handleCornerShift, +handleCornerShift),
	# 			wx.Point(-handleCornerShift, -handleCornerShift),
	# 			wx.Point(multiplier * lineLength - handleCornerShift, -multiplier * lineLength - handleCornerShift),
	# 			wx.Point(multiplier * lineLength + handleCornerShift, -multiplier * lineLength + handleCornerShift),
	# 			]
	# 	mem.DrawPolygon(handlePolygon2, multiplier * lineStartBase,
	# 					multiplier * (x - lineStartBase))
	#
	# 	# Stop drawing on the bitmap before possibly calling RescaleBitmap()
	# 	# below.
	# 	mem.SelectObject(wx.NullBitmap)
	#
	# 	# ===============================================================================
	# 	# end drawing code
	# 	# ===============================================================================
	#
	# 	#if multiplier != 1:
	# 	self.RescaleBitmap(bitmap, wx.Size(x, y))
	#
	# 	return bitmap
	
	def Replace(self, from_, to_, value):
		"""
		Replaces the text starting at the first position up to (but not including) the character at the last position with the given text.

		This function puts the current insertion point position at to as a side effect.
		
		:param from_: The first position
		:type from_:	int
		:param to_: The last position
		:type to_:	int
		:param value: The value to replace the existing text with.
		:type value: str
		"""
		
		self.m_text.Replace(from_, to_, value)
	
	@staticmethod
	def RescaleBitmap(bmp, sizeNeeded):
		"""

		:param bmp:
		:type bmp: wx.Bitmap
		:param sizeNeeded:
		:type sizeNeeded: wx.Size
		"""
		
		if not sizeNeeded.IsFullySpecified():
			raise ValueError("New size must be given")
		
		img = bmp.ConvertToImage()
		img.Rescale(sizeNeeded.x, sizeNeeded.y)
		bmp = wx.Bitmap(img)
	
	#
	# if wx.Platform in {"__WXMSW__", "__WXOSX__"}:
	# 	newBmp.UseAlpha(bmp.HasAlpha())
	#
	# dc(newBmp)
	# scX = sizeNeeded.GetWidth() / bmp.GetWidth()
	# scY = sizeNeeded.GetHeight() / bmp.GetHeight()
	# dc.SetUserScale(scX, scY)
	# dc.DrawBitmap(bmp, 0, 0)
	#
	# bmp = newBmp
	
	def SelectAll(self):
		"""Selects all text in the control."""
		
		self.m_text.SelectAll()
		
	def SelectNone(self):
		"""Deselects selected text in the control."""
		
		self.m_text.SelectNone()
	
	def SetClearBitmap(self, bitmap):
		"""
		
		:param bitmap:
		:type bitmap:	 wxBitmap
		"""
		
		if bitmap.IsOk:
			self.m_clearBitmap = bitmap
			if self.m_clearButton:
				self.m_clearButton.SetBitmapLabel(self.m_clearBitmap)
	
	def SetDefaultStyle(self, style):
		"""
		Changes the default style to use for the new text which is going to be added to the control.
		
		This applies both to the text added programmatically using WriteText or AppendText and to the text entered by the user interactively.
		
		If either of the font, foreground, or background colour is not set in style, the values of the previous default style are used for them. If the previous default style didn’t set them neither, the global font or colours of the text control itself are used as fall back.
		
		However if the style parameter is the default wx.TextAttr, then the default style is just reset (instead of being combined with the new style which wouldn’t change it at all).
		
		:param style: The style for the new text
		:type style: wx.TextAttr

		:return: True on success, False if an error occurred (this may also mean that the styles are not supported under this platform).
		:rtype: bool
		"""
		
		return self.m_text.SetDefaultStyle(style)
	
	def SetEditable(self, editable):
		"""
		Makes the text item editable or read-only, overriding the wx.TE_READONLY flag.
		
		:param editable: If True, the control is editable. If False, the control is read-only.
		:type editable:	bool
		"""
		
		self.m_text.SetEditable(editable)
	
	def SetInsertionPoint(self, pos):
		"""
		Sets the insertion point at the given position.
		
		:param pos: Position to set, in the range from 0 to GetLastPosition inclusive.
		:type pos: int
		"""
		
		self.m_text.SetInsertionPoint(pos)
	
	def SetInsertionPointEnd(self):
		"""
		Sets the insertion point at the end of the text control.

		This is equivalent to calling wx.TextCtrl.SetInsertionPoint with wx.TextCtrl.GetLastPosition argument.
		"""
		
		self.m_text.SetInsertionPointEnd()
	
	# SetMargins
	
	def SetMaxLength(self, len):
		"""
		This function sets the maximum number of characters the user can enter into the control.
		
		In other words, it allows limiting the text value length to len not counting the terminating NUL character.
		
		If len is 0, the previously set max length limit, if any, is discarded and the user may enter as much text as the underlying native text control widget supports (typically at least 32Kb). If the user tries to enter more characters into the text control when it already is filled up to the maximal length, a wxEVT_TEXT_MAXLEN event is sent to notify the program about it (giving it the possibility to show an explanatory message, for example) and the extra input is discarded.
		
		Note that in wxGTK this function may only be used with single line text controls.

		:param len:
		:type len:	long
		"""
		
		self.m_text.SetMaxLength(len)
	
	def SetModified(self, modified):
		"""
		Marks the control as being modified by the user or not.

		:param modified:
		:type modified: bool
		"""
		
		self.m_text.SetModified(modified)
	
	def SetSelection(self, from_, to_):
		"""
		Selects the text starting at the first position up to (but not including) the character at the last position.
	
		If both parameters are equal to -1 all text in the control is selected.
		
		Notice that the insertion point will be moved to from by this function.

		:param from_: The first position
		:type from_: int
		:param to_: The last position
		:type to_: int
		"""
		
		self.m_text.SetSelection(from_, to_)
	
	def SetStyle(self, start, end, style):
		"""
		Changes the style of the given range.

		If any attribute within style is not set, the corresponding attribute from GetDefaultStyle is used.

		:param start: The start of the range to change.
		:type start: int
		:param end: The end of the range to change.
		:type end: int
		:param style: The new style for the range.
		:type style: wx.TextAttr

		:return: True on success, False if an error occurred (this may also mean that the styles are not supported under this platform).
		:rtype: bool
		"""
		
		return self.m_text.SetStyle(start, end, style)
	
	def SetValue(self, value):
		"""
		Sets the new text control value.

		It also marks the control as not-modified which means that IsModified() would return False immediately after the call to SetValue .
		
		The insertion point is set to the start of the control (i.e. position 0) by this function unless the control value doesn’t change at all, in which case the insertion point is left at its original position.
		
		Note that, unlike most other functions changing the controls values, this function generates a wxEVT_TEXT event. To avoid this you can use ChangeValue instead.

		:param value: The new value to set. It may contain newline characters if the text control is multi-line.
		:type value: str
		"""
	
		self.m_text.SetValue(value)
	
	def ShowPosition(self, pos):
		"""
		Makes the line containing the given position visible.
		
		:param pos: The position that should be visible.
		:type pos: int
		"""
		
		self.m_text.ShowPosition(pos)
	
	def ShouldInheritColours(self):
			"""
			
			:rtype: bool
			
			:return:
			:rtype:
			"""
		
			return True
	
	def Undo(self):
		"""
		If there is an undo facility and the last operation can be undone, undoes the last operation.
		
		Does nothing if there is no undo facility.
		"""
		
		self.m_text.Undo()
	
	def WriteText(self, text):
		"""
		Writes the text into the text control at the current insertion position.
		
		:param text: Text to write to the text control
		:type text: str
		"""
		
		self.m_text.WriteText(text)
	
	def XYToPosition(self, x, y):
		"""
		Converts the given zero based column and line number to a position.

		:param x: The column number
		:type x: int
		:param y: The line number
		:type y: int

		:rtype: int
		"""
		
		return self.m_text.XYToPosition(x, y)



# Copyright (C) 2019 Dominic Davis-Foster <dominic@davis-foster.co.uk>
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

# This script based on https://github.com/rocky/python-uncompyle6/blob/master/__pkginfo__.py

copyright = """
2019 Dominic Davis-Foster <dominic@davis-foster.co.uk>
"""

VERSION = "0.2.4"

modname = "domdf_wxpython_tools"
py_modules = None
entry_points = None

license = 'LGPL3'

short_desc = 'Tools and widgets for wxPython'

classifiers = ['Development Status :: 4 - Beta',
			   'Intended Audience :: Developers',
			   "License :: OSI Approved :: GNU Lesser General Public License v3 (LGPLv3)",
			   'Operating System :: OS Independent',
			   'Programming Language :: Python',
			   'Programming Language :: Python :: 3.6',
			   'Programming Language :: Python :: 3.7',
			   'Programming Language :: Python :: 3.8',
			   'Topic :: Software Development :: Libraries :: Python Modules',
			   ]

author = "Dominic Davis-Foster"
author_email = "dominic@davis-foster.co.uk"
github_username = "domdfcoding"
web = github_url = f"https://github.com/{github_username}/{modname}"

import os.path


def get_srcdir():
	filename = os.path.normcase(os.path.dirname(os.path.abspath(__file__)))
	return os.path.realpath(filename)


srcdir = get_srcdir()


def read(*rnames):
	return open(os.path.join(srcdir, *rnames)).read()


# Get info from files; set: long_description
long_description = read("README.rst") + '\n'

# Get requirements
install_requires = list(filter(None, read("requirements.txt").split("\n")))



#  !/usr/bin/env python
#   -*- coding: utf-8 -*-
#
#  chartpanel.py
"""
A canvas for displaying a chart within a wxPython window
"""
#
#  Copyright 2019-2020 Dominic Davis-Foster <dominic@davis-foster.co.uk>
#
#  Method ``constrain_zoom`` based on https://stackoverflow.com/a/16709952/3092681
#  Copyright 2013 simonb
#  https://stackoverflow.com/users/456805/simonb
#
#  Method ``setup_scrollwheel_zooming`` based on https://stackoverflow.com/a/11562898/3092681
#  Copyright 2012 Thomas A Caswell
#  https://stackoverflow.com/users/380231/tacaswell
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#


# stdlib
import types

# 3rd party
import matplotlib
import numpy
import wx.html2
from matplotlib.backends.backend_wxagg import (
	FigureCanvasWxAgg as FigureCanvas,
	NavigationToolbar2WxAgg as NavigationToolbar,
	)

# this package
from domdf_wxpython_tools.border_config import border_config
from domdf_wxpython_tools.projections import XPanAxes


# Constrain zoom to X axis
matplotlib.projections.register_projection(XPanAxes)


class ChartPanelBase(wx.Panel):
	"""
	Panel that contains a matplotlib plotting window, used for displaying an image.
	The image can be right clicked to bring up a context menu allowing copying, pasting and saving of the image.
	The image can be panned by holding the left mouse button and moving the mouse,
	and zoomed in and out using the scrollwheel on the mouse.
	"""
	
	def __init__(
			self, parent, fig, ax, id=wx.ID_ANY, pos=wx.DefaultPosition, size=wx.DefaultSize,
			style=0, name=wx.PanelNameStr
			):
		"""
		:param parent: The parent window.
		:type parent: wx.Window
		:param fig:
		:type fig:
		:param ax:
		:type ax:
		:param id: An identifier for the panel. wx.ID_ANY is taken to mean a default.
		:type id: wx.WindowID, optional
		:param pos: The panel position. The value ::wxDefaultPosition indicates a default position,
		:type pos: wx.Point, optional
			chosen by either the windowing system or wxWidgets, depending on platform.
		:param size: The panel size. The value ::wxDefaultSize indicates a default size, chosen by
		:type size: wx.Size, optional
			either the windowing system or wxWidgets, depending on platform.
		:param style: The window style. See wxPanel.
		:type style: int, optional
		:param name: Window name.
		:type name: str, optional
		"""
		
		wx.Panel.__init__(self, parent, id, pos, size, style | wx.TAB_TRAVERSAL, name)
		
		self.fig = fig
		self.ax = ax
		
		self.canvas = FigureCanvas(self, wx.ID_ANY, self.fig)
		self._do_layout()
		
		self.toolbar = NavigationToolbar(self.canvas)
		self.toolbar.Hide()
		
		self.Bind(wx.EVT_SIZE, self.on_size_change, self)
		self.Bind(wx.EVT_MAXIMIZE, self.on_size_change)
	
	def setup_ylim_refresher(self, y_data, x_data):
		"""
		Setup the function for updating the ylim whenever the xlim changes.s
		
		:param y_data:
		:type y_data:
		:param x_data:
		:type x_data:
		"""
		
		def update_ylim(*args):
			# print(str(*args).startswith("MPL MouseEvent")) # Pan
			if (str(*args).startswith("XPanAxesSubplot") and self.canvas.toolbar._active != "PAN") or (
					str(*args).startswith("MPL MouseEvent") and self.canvas.toolbar._active != "ZOOM"):  # Zoom, Pan
				# print("updated xlims: ", axes.get_xlim())
				min_x_index = (numpy.abs(x_data - self.ax.get_xlim()[0])).argmin()
				max_x_index = (numpy.abs(x_data - self.ax.get_xlim()[1])).argmin()
				# print(min_x_index, max_x_index)
				
				y_vals_for_range = numpy.take(y_data, [idx for idx in range(min_x_index, max_x_index)])
				# print(max(y_vals_for_range))
				self.ax.set_ylim(bottom=0, top=max(y_vals_for_range) * 1.1)
				self.fig.canvas.draw()
				# print("x-val: {}, y-val:{}
				self.size_change()
		
		self.ax.callbacks.connect('xlim_changed', update_ylim)
		self.fig.canvas.callbacks.connect("button_release_event", update_ylim)
	
	def _do_layout(self):
		# begin wxGlade: ChromatogramPanel.__do_layout
		sizer = wx.FlexGridSizer(1, 2, 0, 0)
		sizer.Add(self.canvas, 1, wx.EXPAND, 0)
		self.SetSizer(sizer)
		sizer.Fit(self)
		self.Layout()
	
	def reset_view(self, *_):
		"""
		Reset the view of the chart
		"""
		
		self.canvas.toolbar.home()
		self.canvas.draw_idle()
	
	def previous_view(self, *_):
		"""
		Go to the previous view of the chart
		"""
		
		self.canvas.toolbar.back()
	
	def zoom(self, enable=True):
		"""
		Enable the Zoom tool
		"""
		
		if enable or (not enable and self.canvas.toolbar._active == "ZOOM"):
			self.canvas.toolbar.zoom()
		self.canvas.Refresh()
	
	def pan(self, enable=True):
		"""
		Enable the Pan tool
		"""
		
		if enable or (not enable and self.canvas.toolbar._active == "PAN"):
			self.canvas.toolbar.pan()
		self.canvas.Refresh()
	
	def configure_borders(self, event=None):
		"""
		Open the 'Configure Borders' dialog
		"""
		
		self.border_config = border_config(self, self.fig)
		self.border_config.Show()
		
		if event:
			event.Skip()
	
	def constrain_zoom(self, key="x"):
		"""
		Constrain zoom to the x axis only

		:param key:
		:type key:
		:return:
		:rtype:
		"""
		def press_zoom(self, event):
			event.key = key
			NavigationToolbar.press_zoom(self, event)
		
		self.fig.canvas.toolbar.press_zoom = types.MethodType(
				press_zoom,
				self.fig.canvas.toolbar
				)
	
	# Other Toolbar Options
	# Save chromatogram as image: save_figure(self, *args)
	# set_cursor(self, cursor)
	# Set the current cursor to one of the :class:`Cursors` enums values.
	
	# If required by the backend, this method should trigger an update in
	# the backend event loop after the cursor is set, as this method may be
	# called e.g. before a long-running task during which the GUI is not
	# updated.
	# set_history_buttons(self)
	# Enable or disable the back/forward button.
	# forward(self, *args)
	# move forward in the view lim stack.
	# print(axes.get_ylim())
	# end of class ChromatogramPanel
	
	def size_change(self):
		"""
		Internal function that runs whenever the window is resized
		"""
		
		# self.canvas.SetMinSize(self.GetSize())
		self.canvas.SetSize(self.GetSize())
		self.Refresh()
		self.canvas.draw()
		self.canvas.Refresh()
	
		# if event.ClassName == "wxSizeEvent":
		# 	event.Skip()
	
	def on_size_change(self, event):
		"""
		Event handler for size change events
		"""
		
		self.size_change()
		# event.Skip()
	
	def setup_scrollwheel_zooming(self, scale=1.1):
		"""
		Allow zooming of the chart with the scrollwheel
		
		:param scale:
		:type scale:
		"""
		
		def zoom_factory(ax, base_scale=1.1):
			def zoom_fun(event):
				# get the current x and y limits
				cur_xlim = ax.get_xlim()
				cur_ylim = ax.get_ylim()
				cur_xrange = (cur_xlim[1] - cur_xlim[0]) * .5
				cur_yrange = (cur_ylim[1] - cur_ylim[0]) * .5
				xdata = event.xdata  # get event x location
				ydata = event.ydata  # get event y location
				if event.button == 'up':
					# deal with zoom in
					scale_factor = 1 / base_scale
				elif event.button == 'down':
					# deal with zoom out
					scale_factor = base_scale
				else:
					# deal with something that should never happen
					scale_factor = 1
					print(event.button)
				# set new limits
				ax.set_xlim([
						xdata - cur_xrange * scale_factor,
						xdata + cur_xrange * scale_factor
						])
				ax.set_ylim([
						ydata - cur_yrange * scale_factor,
						ydata + cur_yrange * scale_factor
						])
				self.canvas.draw()  # force re-draw
			
			fig = ax.get_figure()  # get the figure of interest
			# attach the call back
			fig.canvas.mpl_connect('scroll_event', zoom_fun)
			
			# return the function
			return zoom_fun
		
		self.__zoom_factory = zoom_factory(self.ax, base_scale=scale)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  __init__.py
#
#  Copyright 2019-2020 Dominic Davis-Foster <dominic@davis-foster.co.uk>
#
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#

from domdf_wxpython_tools.border_config import border_config as BorderConfigDialog
from domdf_wxpython_tools.chartpanel import ChartPanelBase
from domdf_wxpython_tools.clearable_textctrl import ClearableTextCtrl
from . import ColourPickerPanel  # TODO: ColourPickerPanel
from domdf_wxpython_tools.dialogs import (
	file_dialog_wildcard, file_dialog_multiple, file_dialog, FloatEntryDialog, IntEntryDialog, Wildcards,
	)
from domdf_wxpython_tools.editable_listbox import EditableListBox, CleverListCtrl, EditableNumericalListBox
from domdf_wxpython_tools.events import PayloadEvent, SimpleEvent
from domdf_wxpython_tools.filebrowsectrl import FileBrowseCtrl, FileBrowseCtrlWithHistory, DirBrowseCtrl
from domdf_wxpython_tools.icons import get_button_icon, get_toolbar_icon, GetStockBitmap, GetStockToolbarBitmap
from domdf_wxpython_tools.imagepanel import (
	ID_ImagePanel_Copy_Image, ID_ImagePanel_Delete_Image, ID_ImagePanel_Load_Image,
	ID_ImagePanel_Paste_Image, ID_ImagePanel_Reset_View, ID_ImagePanel_Save_Image,
	EVT_IMAGE_PANEL_CHANGED, ImagePanel,
	)
from domdf_wxpython_tools.keyboard import gen_keymap, NAVKEYS
from . import list_dialog  # TODO: list_dialog
from domdf_wxpython_tools.logctrl import LogCtrl
from . import panel_listctrl
from . import picker  # TODO: picker
from domdf_wxpython_tools.projections import XPanAxes, XPanAxes_NoZoom, NoZoom
from domdf_wxpython_tools.style_picker import style_picker as StylePickerDialog
from domdf_wxpython_tools.style_picker import colour_picker as ColourPickerDialog
from . import StylePickerPanel  # TODO: StylePickerPanel
from domdf_wxpython_tools.tabbable_textctrl import TabbableTextCtrl
from domdf_wxpython_tools.timer_thread import Timer
from domdf_wxpython_tools.utils import toggle, coming_soon, collapse_label, generate_faces
from domdf_wxpython_tools.validators import ValidatorBase, FloatValidator, CharValidator
# TODO: WebView


__all__ = [
		"BorderConfigDialog",
		"ChartPanelBase",
		"ClearableTextCtrl",
		"ColourPickerPanel",
		"file_dialog_wildcard",
		"file_dialog_multiple",
		"file_dialog",
		"FloatEntryDialog",
		"IntEntryDialog",
		"Wildcards",
		"PayloadEvent",
		"SimpleEvent",
		"FileBrowseCtrl",
		"FileBrowseCtrlWithHistory",
		"DirBrowseCtrl",
		"get_button_icon",
		"get_toolbar_icon",
		"GetStockBitmap",
		"GetStockToolbarBitmap",
		"ID_ImagePanel_Copy_Image",
		"ID_ImagePanel_Delete_Image",
		"ID_ImagePanel_Load_Image",
		"ID_ImagePanel_Paste_Image",
		"ID_ImagePanel_Reset_View",
		"ID_ImagePanel_Save_Image",
		"EVT_IMAGE_PANEL_CHANGED",
		"ImagePanel",
		"gen_keymap",
		"NAVKEYS",
		"list_dialog",
		"LogCtrl",
		"picker",
		"panel_listctrl",
		"XPanAxes",
		"XPanAxes_NoZoom",
		"NoZoom",
		"style_picker",
		"StylePickerPanel",
		"TabbableTextCtrl",
		"Timer",
		"toggle",
		"coming_soon",
		"collapse_label",
		"generate_faces",
		"ValidatorBase",
		"FloatValidator",
		"CharValidator",
		"WebView",
		"list_dialog",
		"picker",
		"style_picker",
		"StylePickerPanel",
		"EditableListBox",
		"CleverListCtrl",
		"EditableNumericalListBox",
		"StylePickerDialog",
		"ColourPickerDialog",
		"__author__",
		"__version__",
		"__copyright__"
]

__author__ = "Dominic Davis-Foster"
__copyright__ = "Copyright 2019-2020 Dominic Davis-Foster"

__license__ = "LGPL"
__version__ = "0.2.4"
__email__ = "dominic@davis-foster.co.uk"

